
import binascii
import unittest

from pycoin.satoshi import stackops


def b2h(b):
    return binascii.hexlify(b).decode("utf8")


class StackOpsTest(unittest.TestCase):
    def test_do_OP_2DROP(self):
        s = [1, 2, 3]
        stackops.do_OP_2DROP(s)
        self.assertEqual(s, [1])

    def test_do_OP_2DUP(self):
        s = [1, 2]
        stackops.do_OP_2DUP(s)
        self.assertEqual(s, [1, 2, 1, 2])

    def test_do_OP_3DUP(self):
        s = [1, 2, 3]
        stackops.do_OP_3DUP(s)
        self.assertEqual(s, [1, 2, 3, 1, 2, 3])

    def test_do_OP_2OVER(self):
        s = [1, 2, 3, 4]
        stackops.do_OP_2OVER(s)
        self.assertEqual(s, [1, 2, 3, 4, 1, 2])

    def test_do_OP_2ROT(self):
        s = [1, 2, 3, 4, 5, 6]
        stackops.do_OP_2ROT(s)
        self.assertEqual(s, [3, 4, 5, 6, 1, 2])

    def test_do_OP_2SWAP(self):
        s = [1, 2, 3, 4]
        stackops.do_OP_2SWAP(s)
        self.assertEqual(s, [3, 4, 1, 2])

    def test_do_OP_IFDUP(self):
        s = [1, 2]
        stackops.do_OP_IFDUP(s)
        self.assertEqual(s, [1, 2, 2])
        s = [1, 2, 0]
        stackops.do_OP_IFDUP(s)
        self.assertEqual(s, [1, 2, 0])

    def test_do_OP_DROP(self):
        s = [1, 2]
        stackops.do_OP_DROP(s)
        self.assertEqual(s, [1])

    def test_do_OP_DUP(self):
        s = [1, 2]
        stackops.do_OP_DUP(s)
        self.assertEqual(s, [1, 2, 2])

    def test_do_OP_NIP(self):
        s = [1, 2]
        stackops.do_OP_NIP(s)
        self.assertEqual(s, [2])

    def test_do_OP_OVER(self):
        s = [1, 2]
        stackops.do_OP_OVER(s)
        self.assertEqual(s, [1, 2, 1])

    def test_do_OP_ROT(self):
        s = [1, 2, 3]
        stackops.do_OP_ROT(s)
        self.assertEqual(s, [2, 3, 1])

    def test_do_OP_SWAP(self):
        s = [1, 2, 3]
        stackops.do_OP_SWAP(s)
        self.assertEqual(s, [1, 3, 2])

    def test_do_OP_TUCK(self):
        s = [1, 2, 3]
        stackops.do_OP_TUCK(s)
        self.assertEqual(s, [1, 3, 2, 3])

    def test_do_OP_CAT(self):
        s = ["foo", "bar"]
        stackops.do_OP_CAT(s)
        self.assertEqual(s, ['foobar'])

    def test_do_OP_RIPEMD160(self):
        s = [b'foo']
        stackops.do_OP_RIPEMD160(s)
        self.assertEqual(len(s), 1)
        self.assertEqual(b2h(s[0]), "42cfa211018ea492fdee45ac637b7972a0ad6873")

    def test_do_OP_SHA1(self):
        s = [b'foo']
        stackops.do_OP_SHA1(s)
        self.assertEqual(len(s), 1)
        self.assertEqual(b2h(s[0]), "0beec7b5ea3f0fdbc95d0dd47f3c5bc275da8a33")

    def test_do_OP_SHA256(self):
        s = [b'foo']
        stackops.do_OP_SHA256(s)
        self.assertEqual(len(s), 1)
        self.assertEqual(b2h(s[0]), "2c26b46b68ffc68ff99b453c1d30413413422d706483bfa0f98a5e886266e7ae")

    def test_do_OP_HASH160(self):
        s = [b'foo']
        stackops.do_OP_HASH160(s)
        self.assertEqual(len(s), 1)
        self.assertEqual(b2h(s[0]), "e1cf7c8103476b6d7fe9e4979aa10e7c531fcf42")

    def test_do_OP_HASH256(self):
        s = [b'foo']
        stackops.do_OP_HASH256(s)
        self.assertEqual(len(s), 1)
        self.assertEqual(b2h(s[0]), "c7ade88fc7a21498a6a5e5c385e1f68bed822b72aa63c4a9a48a02c2466ee29e")


if __name__ == "__main__":
    unittest.main()



import unittest

from pycoin.networks.registry import network_for_netcode


def make_tests_for_netcode(netcode):
    network = network_for_netcode(netcode)

    address_for_script = network.address.for_script
    script_for_p2pkh = network.contract.for_p2pkh
    script_for_p2pk = network.contract.for_p2pk
    script_for_nulldata = network.contract.for_nulldata

    Tx = network.tx

    class AddressForScriptTest(unittest.TestCase):

        def test_script_type_pay_to_address(self):
            for se in range(1, 100):
                key = network.keys.private(secret_exponent=se)
                for b in [True, False]:
                    addr = key.address(is_compressed=b)
                    sc = script_for_p2pkh(key.hash160(is_compressed=b))
                    afs_address = address_for_script(sc)
                    self.assertEqual(afs_address, addr)

        def test_solve_pay_to_address(self):
            for se in range(1, 10):
                key = network.keys.private(secret_exponent=se)
                for b in [True, False]:
                    addr = key.address(is_compressed=b)
                    script = script_for_p2pkh(key.hash160(is_compressed=b))
                    afs_address = address_for_script(script)
                    self.assertEqual(afs_address, addr)
                    hl = network.tx.solve.build_hash160_lookup([se])
                    tx = Tx(1, [], [Tx.TxOut(100, script)])
                    tx.sign(hash160_lookup=hl)
                    afs_address = address_for_script(tx.txs_out[0].puzzle_script())
                    self.assertEqual(afs_address, addr)

        def test_script_type_pay_to_public_pair(self):
            for se in range(1, 100):
                key = network.keys.private(secret_exponent=se)
                for b in [True, False]:
                    addr = key.address(is_compressed=b)
                    sc = script_for_p2pk(key.sec(is_compressed=b))
                    afs_address = address_for_script(sc)
                    self.assertEqual(afs_address, addr)

        def test_solve_pay_to_public_pair(self):
            for se in range(1, 10):
                key = network.keys.private(secret_exponent=se)
                for b in [True, False]:
                    addr = key.address(is_compressed=b)
                    script = script_for_p2pk(key.sec(is_compressed=b))
                    afs_address = address_for_script(script)
                    self.assertEqual(afs_address, addr)
                    hl = network.tx.solve.build_hash160_lookup([se])
                    tx = Tx(1, [], [Tx.TxOut(100, script)])
                    tx.sign(hash160_lookup=hl)
                    afs_address = address_for_script(tx.txs_out[0].puzzle_script())
                    self.assertEqual(afs_address, addr)

        def test_weird_tx(self):
            # this is from tx 12a8d1d62d12307eac6e62f2f14d7e826604e53c320a154593845aa7c8e59fbf
            afs_address = address_for_script(b'Q')
            self.assertNotEqual(afs_address, None)
            self.assertEqual(afs_address, "???")

        def test_issue_225(self):
            script = script_for_nulldata(b"foobar")
            tx_out = Tx.TxOut(1, script)
            address = address_for_script(tx_out.puzzle_script())
            self.assertEqual(address, "(nulldata 666f6f626172)")

    return AddressForScriptTest


for netcode in ["BTC", "LTC", "BCH"]:
    exec("%sTests = make_tests_for_netcode('%s')" % (netcode, netcode))


if __name__ == "__main__":
    unittest.main()



import os

from .intstream import from_bytes

from .Curve import Curve
from .Point import Point

from .rfc6979 import deterministic_generate_k


class Generator(Curve, Point):
    """
    A Generator is a specific point on an elliptic curve that defines a `trapdoor
    function <https://en.wikipedia.org/wiki/Trapdoor_function>`_ from integers to curve points.

    :param p: the prime for the :class:`Curve <pycoin.ecdsa.Curve.Curve>`
    :param a: the a value for the :class:`Curve <pycoin.ecdsa.Curve.Curve>`
    :param b: the b value for the :class:`Curve <pycoin.ecdsa.Curve.Curve>`
    :param basis: a :class:`Point <pycoin.ecdsa.Point.Point>` on the
        given :class:`Curve <pycoin.ecdsa.Curve.Curve>`
    :param order: the order for the :class:`Curve <pycoin.ecdsa.Curve.Curve>`

    The constructor raises :class:`NoSuchPointError` if the point is invalid.
    The point at infinity is ``(x, y) == (None, None)``.
    """
    def __new__(self, p, a, b, basis, order):
        # since Generator extends tuple (via Point), we need to override __new__
        return tuple.__new__(self, basis)

    def __init__(self, p, a, b, basis, order, entropy_f=os.urandom):
        """
        Set up a group with generator basis for the curve y^2 = x^3 + x*a + b (mod p).
        The order is the order of the group (it's generally predetermined for a given curve;
        how it's calculated is complicated).
        The entropy function creates a blinding factor, to mitigate side channel attacks.
        """
        Curve.__init__(self, p, a, b, order)
        Point.__init__(self, basis[0], basis[1], self)
        self._powers = []
        Gp = self
        for _ in range(256):
            self._powers.append(Gp)
            Gp += Gp
        assert p % 4 == 3, "p % 4 must be 3 due to modular_sqrt optimization"
        self._mod_sqrt_power = (p + 1) // 4
        self._blinding_factor = from_bytes(entropy_f(32)) % self._order
        self._minus_blinding_factor_g = self.raw_mul(-self._blinding_factor)

    def modular_sqrt(self, a):
        """
        :return: n where ``n * n == a (mod p) for the curve's prime p``.
            If no such n exists, an arbitrary value will be returned.
        """
        return pow(a, self._mod_sqrt_power, self._p)

    def inverse(self, a):
        ":return: n where ``a * n == 1 (mod p) for the curve's prime p``."
        return self.inverse_mod(a, self._order)

    def points_for_x(self, x):
        """
        :param: x: an integer x coordinate
        :return: (p0, p1) where each p is a :class:`Point` with given x coordinate,
            and p0's y value is even.

        To get a point with particular parity, use::
            points_for_x(x)[1 if is_y_supposed_to_be_odd else 0]
        """
        p = self._p
        alpha = (pow(x, 3, p) + self._a * x + self._b) % p
        y0 = self.modular_sqrt(alpha)
        if y0 == 0:
            raise ValueError("no y value for %d" % x)
        p0, p1 = [self.Point(x, _) for _ in (y0, p - y0)]
        if y0 & 1 == 0:
            return (p0, p1)
        return (p1, p0)

    def possible_public_pairs_for_signature(self, value, signature, y_parity=None):
        """
        :param: value: an integer value
        :param: signature: an ``(r, s)`` pair of integers representing an ecdsa signature of ``value``
        :param: y_parity: (optional) for a given value and signature, there are either two points
            that sign it, or none if the signature is invalid. One of the points has an even y
            value, the other an odd. If this parameter is set, only points whose y value matches
            this value will be returned in the list.

        :return: a list of :class:`Point <pycoin.ecdsa.Point.Point>` objects p where each p is
            a possible public key for which ``signature`` correctly signs the given ``value``.
            If something goes wrong, this list will be empty.
        """
        r, s = signature

        try:
            points = self.points_for_x(r)
        except ValueError:
            return []

        if y_parity is not None:
            if y_parity & 1:
                points = points[1:]
            else:
                points = points[:1]

        inv_r = self.inverse(r)
        s_over_r = s * inv_r
        minus_E_over_r = -(inv_r * value) * self
        try:
            return [s_over_r * p + minus_E_over_r for p in points]
        except ValueError:
            return []

    def raw_mul(self, e):
        """
        :param: e: an integer value
        :returns: e * self

        This method uses a precomputed table as an optimization.
        """
        e %= self._order
        P = self._infinity
        for bit in range(256):
            # add the power of the generator every time to make it more time-deterministic
            a = [P, P + self._powers[bit]]
            # choose the correct result
            P = a[e & 1]
            e >>= 1
        return P

    def __mul__(self, e):
        """Multiply the generator by an integer. Uses the blinding factor."""
        return self.raw_mul(e + self._blinding_factor) + self._minus_blinding_factor_g

    def __rmul__(self, e):
        """Multiply the generator by an integer."""
        return self.__mul__(e)

    def verify(self, public_pair, val, sig):
        """
        :param: public_pair: a :class:`Point <pycoin.ecdsa.Point.Point>` on the curve
        :param: val: an integer value
        :param: sig: a pair of integers ``(r, s)`` representing an ecdsa signature

        :returns: True if and only if the signature ``sig`` is a valid signature
            of ``val`` using ``public_pair`` public key.
        """
        order = self._order
        if val == 0:
            return False
        r, s = sig
        if r < 1 or r >= order or s < 1 or s >= order:
            return False
        s_inverse = self.inverse(s)
        u1 = val * s_inverse
        u2 = r * s_inverse
        point = u1 * self + u2 * self.Point(*public_pair)
        v = point[0] % order
        return v == r

    def sign_with_recid(self, secret_exponent, val, gen_k=None):
        """
        :param: secret_exponent: a :class:`Point <pycoin.ecdsa.Point.Point>` on the curve
        :param: val: an integer value
        :param: gen_k: a function generating __k values__

        :returns: a tuple of integers ``(r, s, recid)`` where ``(r, s)`` represents an ecdsa
            signature of ``val`` with public key ``self * secret_exponent``; and ``recid``
            is the **recovery id**, a number from 0-3 used to eliminate ambiguity about
            which public key signed the value.

        If gen_k is set, it will be called with (n, secret_exponent, val), and an unguessable
        K value should be returned. Otherwise, the default K value, generated according
        to rfc6979 will be used.
        """
        if val == 0:
            raise ValueError()
        if gen_k is None:
            gen_k = deterministic_generate_k
        n = self._order
        k = gen_k(n, secret_exponent, val)
        while True:
            p1 = k * self
            r = p1[0] % n
            s = (self.inverse(k) * (val + (secret_exponent * r) % n)) % n
            if r != 0 and s != 0:
                recid = p1[1] & 1
                if p1[0] > n:
                    recid += 2
                return r, s, recid
            k += 1

    def sign(self, secret_exponent, val, gen_k=None):
        """
        :param: secret_exponent: a :class:`Point <pycoin.ecdsa.Point.Point>` on the curve
        :param: val: an integer value
        :param: gen_k: a function generating __k values__

        :returns: a pair of integers ``(r, s)`` represents an ecdsa signature of ``val``
            with public key ``self * secret_exponent``.

        If gen_k is set, it will be called with (n, secret_exponent, val), and an unguessable
        K value should be returned. Otherwise, the default K value, generated according
        to rfc6979 will be used.
        """
        return self.sign_with_recid(secret_exponent, val, gen_k)[0:2]



import ctypes
import os
import warnings

from ctypes import (
    byref, c_byte, c_int, c_uint, c_char_p, c_size_t, c_void_p, create_string_buffer, CFUNCTYPE, POINTER
)

from pycoin.encoding.bytes32 import from_bytes_32, to_bytes_32
from pycoin.intbytes import iterbytes


SECP256K1_FLAGS_TYPE_MASK = ((1 << 8) - 1)
SECP256K1_FLAGS_TYPE_CONTEXT = (1 << 0)
SECP256K1_FLAGS_TYPE_COMPRESSION = (1 << 1)
# /** The higher bits contain the actual data. Do not use directly. */
SECP256K1_FLAGS_BIT_CONTEXT_VERIFY = (1 << 8)
SECP256K1_FLAGS_BIT_CONTEXT_SIGN = (1 << 9)
SECP256K1_FLAGS_BIT_COMPRESSION = (1 << 8)

# /** Flags to pass to secp256k1_context_create. */
SECP256K1_CONTEXT_VERIFY = (SECP256K1_FLAGS_TYPE_CONTEXT | SECP256K1_FLAGS_BIT_CONTEXT_VERIFY)
SECP256K1_CONTEXT_SIGN = (SECP256K1_FLAGS_TYPE_CONTEXT | SECP256K1_FLAGS_BIT_CONTEXT_SIGN)
SECP256K1_CONTEXT_NONE = (SECP256K1_FLAGS_TYPE_CONTEXT)

SECP256K1_FLAGS_BIT_COMPRESSION = (1 << 8)
SECP256K1_EC_COMPRESSED = (SECP256K1_FLAGS_TYPE_COMPRESSION | SECP256K1_FLAGS_BIT_COMPRESSION)
SECP256K1_EC_UNCOMPRESSED = (SECP256K1_FLAGS_TYPE_COMPRESSION)


def load_library():
    try:
        PYCOIN_LIBSECP256K1_PATH = os.getenv("PYCOIN_LIBSECP256K1_PATH")
        library_path = PYCOIN_LIBSECP256K1_PATH or ctypes.util.find_library('libsecp256k1')

        secp256k1 = ctypes.cdll.LoadLibrary(library_path)

        secp256k1.secp256k1_context_create.argtypes = [c_uint]
        secp256k1.secp256k1_context_create.restype = c_void_p

        secp256k1.secp256k1_context_randomize.argtypes = [c_void_p, c_char_p]
        secp256k1.secp256k1_context_randomize.restype = c_int

        secp256k1.secp256k1_ec_pubkey_create.argtypes = [c_void_p, c_void_p, c_char_p]
        secp256k1.secp256k1_ec_pubkey_create.restype = c_int

        secp256k1.secp256k1_ecdsa_sign.argtypes = [c_void_p, c_char_p, c_char_p, c_char_p, c_void_p, c_void_p]
        secp256k1.secp256k1_ecdsa_sign.restype = c_int

        secp256k1.secp256k1_ecdsa_verify.argtypes = [c_void_p, c_char_p, c_char_p, c_char_p]
        secp256k1.secp256k1_ecdsa_verify.restype = c_int

        secp256k1.secp256k1_ec_pubkey_parse.argtypes = [c_void_p, c_char_p, c_char_p, c_int]
        secp256k1.secp256k1_ec_pubkey_parse.restype = c_int

        secp256k1.secp256k1_ec_pubkey_serialize.argtypes = [c_void_p, c_char_p, c_void_p, c_char_p, c_uint]
        secp256k1.secp256k1_ec_pubkey_serialize.restype = c_int

        secp256k1.secp256k1_ecdsa_signature_parse_compact.argtypes = [c_void_p, c_char_p, c_char_p]
        secp256k1.secp256k1_ecdsa_signature_parse_compact.restype = c_int

        secp256k1.secp256k1_ecdsa_signature_serialize_compact.argtypes = [c_void_p, c_char_p, c_char_p]
        secp256k1.secp256k1_ecdsa_signature_serialize_compact.restype = c_int

        secp256k1.secp256k1_ec_pubkey_tweak_mul.argtypes = [c_void_p, c_char_p, c_char_p]
        secp256k1.secp256k1_ec_pubkey_tweak_mul.restype = c_int

        secp256k1.ctx = secp256k1.secp256k1_context_create(SECP256K1_CONTEXT_SIGN | SECP256K1_CONTEXT_VERIFY)
        r = secp256k1.secp256k1_context_randomize(secp256k1.ctx, os.urandom(32))
        if r:
            return secp256k1
    except (OSError, AttributeError, TypeError):
        if PYCOIN_LIBSECP256K1_PATH:
            warnings.warn("PYCOIN_LIBSECP256K1_PATH set but libsecp256k1 optimizations not loaded")
        return None


libsecp256k1 = load_library()


class Optimizations:

    def __mul__(self, e):
        e %= self.order()
        if e == 0:
            return self._infinity
        pubkey = create_string_buffer(65)
        libsecp256k1.secp256k1_ec_pubkey_create(libsecp256k1.ctx, pubkey, c_char_p(to_bytes_32(e)))
        pubkey_size = c_size_t(65)
        pubkey_serialized = create_string_buffer(65)
        libsecp256k1.secp256k1_ec_pubkey_serialize(
            libsecp256k1.ctx, pubkey_serialized, byref(pubkey_size), pubkey, SECP256K1_EC_UNCOMPRESSED)
        x = from_bytes_32(pubkey_serialized[1:33])
        y = from_bytes_32(pubkey_serialized[33:])
        return self.Point(x, y)

    def sign(self, secret_exponent, val, gen_k=None):
        nonce_function = None
        if gen_k is not None:
            k_as_bytes = to_bytes_32(gen_k(self.order(), secret_exponent, val))

            def adaptor(nonce32_p, msg32_p, key32_p, algo16_p, data, attempt):
                nonce32_p.contents[:] = list(iterbytes(k_as_bytes))
                return 1
            p_b32 = POINTER(c_byte*32)
            nonce_function = CFUNCTYPE(c_int, p_b32, p_b32, p_b32, POINTER(c_byte*16), c_void_p, c_uint)(adaptor)

        sig = create_string_buffer(64)
        sig_hash_bytes = to_bytes_32(val)
        libsecp256k1.secp256k1_ecdsa_sign(
            libsecp256k1.ctx, sig, sig_hash_bytes, to_bytes_32(secret_exponent), nonce_function, None)
        compact_signature = create_string_buffer(64)
        libsecp256k1.secp256k1_ecdsa_signature_serialize_compact(libsecp256k1.ctx, compact_signature, sig)
        r = from_bytes_32(compact_signature[:32])
        s = from_bytes_32(compact_signature[32:])
        return (r, s)

    def verify(self, public_pair, val, signature_pair):
        sig = create_string_buffer(64)
        input64 = to_bytes_32(signature_pair[0]) + to_bytes_32(signature_pair[1])
        r = libsecp256k1.secp256k1_ecdsa_signature_parse_compact(libsecp256k1.ctx, sig, input64)
        if not r:
            return False
        r = libsecp256k1.secp256k1_ecdsa_signature_normalize(libsecp256k1.ctx, sig, sig)

        public_pair_bytes = b'\4' + to_bytes_32(public_pair[0]) + to_bytes_32(public_pair[1])
        pubkey = create_string_buffer(64)
        r = libsecp256k1.secp256k1_ec_pubkey_parse(
            libsecp256k1.ctx, pubkey, public_pair_bytes, len(public_pair_bytes))
        if not r:
            return False

        return 1 == libsecp256k1.secp256k1_ecdsa_verify(libsecp256k1.ctx, sig, to_bytes_32(val), pubkey)

    def multiply(self, p, e):
        """Multiply a point by an integer."""
        e %= self.order()
        if p == self._infinity or e == 0:
            return self._infinity
        pubkey = create_string_buffer(64)
        public_pair_bytes = b'\4' + to_bytes_32(p[0]) + to_bytes_32(p[1])
        r = libsecp256k1.secp256k1_ec_pubkey_parse(
            libsecp256k1.ctx, pubkey, public_pair_bytes, len(public_pair_bytes))
        if not r:
            return False
        r = libsecp256k1.secp256k1_ec_pubkey_tweak_mul(libsecp256k1.ctx, pubkey, to_bytes_32(e))
        if not r:
            return self._infinity

        pubkey_serialized = create_string_buffer(65)
        pubkey_size = c_size_t(65)
        libsecp256k1.secp256k1_ec_pubkey_serialize(
            libsecp256k1.ctx, pubkey_serialized, byref(pubkey_size), pubkey, SECP256K1_EC_UNCOMPRESSED)
        x = from_bytes_32(pubkey_serialized[1:33])
        y = from_bytes_32(pubkey_serialized[33:])
        return self.Point(x, y)


def create_LibSECP256K1Optimizations():
    class noop:
        pass

    native = os.getenv("PYCOIN_NATIVE")
    if native and native.lower() != "secp256k1":
        return noop

    if not libsecp256k1:
        return noop

    return Optimizations


LibSECP256K1Optimizations = create_LibSECP256K1Optimizations()



import unittest
import os
import tempfile

from pycoin.cmds import tx
from pycoin.encoding.hexbytes import h2b
from pycoin.symbols.btc import network

from .ToolTest import ToolTest


# BRAIN DAMAGE
Tx = network.tx


class TxTest(ToolTest):

    @classmethod
    def setUpClass(cls):
        cls.parser = tx.create_parser()
        cls.tool_name = "tx"

    def test_tx_fetch_unspent(self):
        P2 = "76a914b3407d4b4d1fca87fb930abe3fa6c2baed6e6fd888ac"
        FETCH_UNSPENT_EXPECT = [
            "5fd3d8275afb5b5cc202ae8480daefa4fe16d0cf480ce78545d6dc06c6fb101a/1/%s/1/0/0/0" % P2,
            "b9a84cffd3766bb642a697065b477eed032e36c377db80faac79b18e61b43b0d/1/%s/1/0/0/0" % P2,
            "0986d70aaa03213135998cf1a9b8a33012c033c6607584e84b8ae33d49fadce3/0/%s/1/0/0/0" % P2,
            "d658ab87cc053b8dbcfd4aa2717fd23cc3edfe90ec75351fadd6a0f7993b461d/7/%s/911/0/0/0" % P2,
        ]
        output = self.launch_tool("tx -i 1HLoD9E4SDFFPDiYfNYnkBLQ85Y51J3Zb1").split("\n")
        for k in FETCH_UNSPENT_EXPECT:
            self.assertIn(k, output)

    def test_cache_tx(self):
        the_dir = self.set_cache_dir()
        tx = Tx.from_hex(
            "01000000010000000000000000000000000000000000000000000000000000000000000000"
            "ffffffff0704ffff001d0104ffffffff0100f2052a0100000043410496b538e853519c726a"
            "2c91e61ec11600ae1390813a627c66fb8be7947be63c52da7589379515d4e0a604f8141781"
            "e62294721166bf621e73a82cbf2342c858eeac00000000")
        self.launch_tool("tx -C %s --db %s" % (tx.id(), tx.as_hex()), env=dict(PYCOIN_CACHE_DIR=the_dir))
        self.assertTrue(os.path.exists(os.path.join(the_dir, "txs", "%s_tx.bin" % tx.id())))

    def test_pay_to_script_file(self):
        the_dir = self.set_cache_dir()
        p2sh_file = tempfile.NamedTemporaryFile()
        p2sh_file.write(
            "52210279be667ef9dcbbac55a06295ce870b07029bfcdb2dce28d959f2815b16f817982102c60"
            "47f9441ed7d6d3045406e95c07cd85c778e4b8cef3ca7abac09b95c709ee52102f9308a019258"
            "c31049344f85f89d5229b531c845836f99b08601f113bce036f953ae\n".encode("utf8"))
        p2sh_file.write(
            "53210279be667ef9dcbbac55a06295ce870b07029bfcdb2dce28d959f2815b16f817982102c60"
            "47f9441ed7d6d3045406e95c07cd85c778e4b8cef3ca7abac09b95c709ee52102f9308a019258"
            "c31049344f85f89d5229b531c845836f99b08601f113bce036f953ae\n".encode("utf8"))
        p2sh_file.flush()
        tx_source_hex = (
            "01000000010000000000000000000000000000000000000000000000000000000000000000ffffffff0"
            "0ffffffff0200f902950000000017a91415fc0754e73eb85d1cbce08786fadb7320ecb8dc8700f90295"
            "0000000017a914594f349df0bac3084ffea8a477bba5f03dcd45078700000000")
        self.launch_tool("tx -C %s" % tx_source_hex)
        tx_to_sign = (
            "01000000020a316ea8980ef9ba02f4e6637c88229bf059f39b06238d48d06a8e"
            "f672aea2bb0000000000ffffffff0a316ea8980ef9ba02f4e6637c88229bf059"
            "f39b06238d48d06a8ef672aea2bb0100000000ffffffff01f0ca052a01000000"
            "1976a914751e76e8199196d454941c45d1b3a323f1433bd688ac0000000000f9"
            "02950000000017a91415fc0754e73eb85d1cbce08786fadb7320ecb8dc8700f9"
            "02950000000017a914594f349df0bac3084ffea8a477bba5f03dcd450787")
        wifs = ' '.join(network.keys.private(_).wif() for _ in (1, 2, 3))
        signed = tempfile.mktemp(suffix=".hex")
        self.launch_tool("tx -a -P %s --db %s %s %s -o %s" % (
            p2sh_file.name, tx_source_hex, tx_to_sign, wifs, signed), env=dict(PYCOIN_CACHE_DIR=the_dir))
        tx = Tx.from_hex(open(signed).read())
        self.assertEqual(tx.id(), "9d991ddccf77e33cb4584e4fc061a36da0da43589232b2e78a1aa0748ac3254b")

    def test_tx_with_gpg(self):
        # binary data with GPG-encrypted WIF KwDiBf89QgGbjEhKnhXJuH7LrciVrZi3qYjgd9M7rFU73sVHnoWn for secret exponent 1
        WIF_1_GPG = h2b(
            "8c0d040303026c3030b7518a94eb60c950bc87ab26f0604a37f247f74f88deda10b180bb807"
            "2879b728b8f056808baea0c8e511e7cf2eba77cce937d2f69a67a79e163bf70b57113d27cb6"
            "a1c2390a1e8069b447c34a7c9b5ba268c2beedd85b50")
        gpg_wif = tempfile.NamedTemporaryFile(suffix=".gpg")
        gpg_wif.write(WIF_1_GPG)
        gpg_wif.flush()
        output_file = tempfile.NamedTemporaryFile(suffix=".hex")
        self.launch_tool(
            args=["tx",
                  "5564224b6c01dbc2bfad89bfd8038bc2f4ca6c55eb660511d7151d71e4b94b6d/0/"
                  "210279be667ef9dcbbac55a06295ce870b07029bfcdb2dce28d959f2815b16f81798ac/5000000000",
                  "1KissFDVu2wAYWPRm4UGh5ZCDU9sE9an8T", "-f", gpg_wif.name, "-g", "--batch --passphrase=foo",
                  "-o", output_file.name])
        d = open(output_file.name).read()
        tx = Tx.from_hex(d)
        self.assertEqual(tx.id(), "c52b0c66cff6147b99acb29389343f6eae68c29faf2186fa8c1613d615b217e8")


def main():
    unittest.main()


if __name__ == "__main__":
    main()



import binascii
import unittest

from pycoin.bloomfilter import filter_size_required, hash_function_count_required, BloomFilter, murmur3
from pycoin.symbols.btc import network

Spendable = network.tx.Spendable

h2b = binascii.unhexlify


class BloomFilterTest(unittest.TestCase):

    def test_filter_size_required(self):
        for ec, fpp, ev in [
            (1, 0.00001, 3),
            (1, 0.00000001, 5),
            (100, 0.000001, 360),
        ]:
            fsr = filter_size_required(ec, fpp)
            self.assertEqual(fsr, ev)

    def test_hash_function_count_required(self):
        for fs, ec, ev in [
            (1, 1, 6),
            (3, 1, 17),
            (5, 1, 28),
            (360, 100, 20),
        ]:
            av = hash_function_count_required(fs, ec)
            self.assertEqual(av, ev)

    def test_BloomFilter(self):
        bf = BloomFilter(20, hash_function_count=5, tweak=127)
        bf.add_hash160(h2b("751e76e8199196d454941c45d1b3a323f1433bd6"))
        tx_hash = h2b("79be667ef9dcbbac55a06295ce870b07029bfcdb2dce28d959f2815b16f81798")
        spendable = Spendable(coin_value=1000, script=b'foo', tx_hash=tx_hash, tx_out_index=1)
        bf.add_spendable(spendable)
        self.assertEqual(bf.filter_bytes, h2b("0000400000000008011130000000101100000000"))

    def test_murmur3(self):
        # test vectors from https://stackoverflow.com/questions/14747343/murmurhash3-test-vectors
        TEST_VECTORS = [
            (b'', 0, 0),
            (b'', 1, 0x514E28B7),
            (b'', 0xffffffff, 0x81F16F39),  # | make sure your seed uses unsigned 32-bit math
            (h2b("FFFFFFFF"), 0, 0x76293B50),  # | make sure 4-byte chunks use unsigned math
            (h2b("21436587"), 0, 0xF55B516B),  # | Endian order. UInt32 should end up as 0x87654321
            (h2b("21436587"), 0x5082EDEE, 0x2362F9DE),  # | Special seed value eliminates initial key with xor
            (h2b("214365"), 0, 0x7E4A8634),  # | Only three bytes. Should end up as 0x654321
            (h2b("2143"), 0, 0xA0F7B07A),  # | Only two bytes. Should end up as 0x4321
            (h2b("21"), 0, 0x72661CF4),  # | Only one byte. Should end up as 0x21
            (h2b("00000000"), 0, 0x2362F9DE),  # | Make sure compiler doesn't see zero and convert to null
            (h2b("000000"), 0, 0x85F0B427),  #
            (h2b("0000"), 0, 0x30F4C306),
            (h2b("00"), 0, 0x514E28B7),
        ]
        for data, seed, expected_value in TEST_VECTORS:
            actual_value = murmur3(data, seed=seed)
            self.assertEqual(expected_value, actual_value)


if __name__ == "__main__":
    unittest.main()



from pycoin.networks.bitcoinish import create_bitcoinish_network

from pycoin.coins.bgold.Tx import Tx as BgoldTx
from pycoin.coins.bgold.Block import Block as BgoldBlock

# fork at block 491407

network = create_bitcoinish_network(
    symbol="BTG", network_name="Bgold", subnet_name="mainnet", tx=BgoldTx, block=BgoldBlock,
    wif_prefix_hex="80", sec_prefix="BTGSEC:", address_prefix_hex="26", pay_to_script_prefix_hex="17",
    bip32_prv_prefix_hex="0488ade4", bip32_pub_prefix_hex="0488B21E",
    magic_header_hex="e1476d44", default_port=8338,
    dns_bootstrap=[
        "eu-dnsseed.bitcoingold-official.org", "dnsseed.bitcoingold.org",
        "dnsseed.btcgpu.org",
    ])



from pycoin.encoding.bytes32 import from_bytes_32
from pycoin.encoding.hash import double_sha256
from pycoin.satoshi.flags import SIGHASH_FORKID

from ..bitcoin.SolutionChecker import BitcoinSolutionChecker


class BgoldSolutionChecker(BitcoinSolutionChecker):

    FORKID_BTG = 79  # atomic number for Au (gold)

    def _signature_hash(self, tx_out_script, unsigned_txs_out_idx, hash_type):
        """
        Return the canonical hash for a transaction. We need to
        remove references to the signature, since it's a signature
        of the hash before the signature is applied.

        tx_out_script: the script the coins for unsigned_txs_out_idx are coming from
        unsigned_txs_out_idx: where to put the tx_out_script
        hash_type: one of SIGHASH_NONE, SIGHASH_SINGLE, SIGHASH_ALL,
        optionally bitwise or'ed with SIGHASH_ANYONECANPAY
        """

        if hash_type & SIGHASH_FORKID != SIGHASH_FORKID:
            raise self.ScriptError()

        return self._signature_for_hash_type_segwit(tx_out_script, unsigned_txs_out_idx, hash_type)

    def _signature_for_hash_type_segwit(self, script, tx_in_idx, hash_type):
        hash_type |= self.FORKID_BTG << 8
        return from_bytes_32(double_sha256(self._segwit_signature_preimage(script, tx_in_idx, hash_type)))



from pycoin.networks.bitcoinish import create_bitcoinish_network


network = create_bitcoinish_network(
    network_name="Litecoin", symbol="LTC", subnet_name="mainnet",
    wif_prefix_hex="b0", sec_prefix="LTCSEC:", address_prefix_hex="30", pay_to_script_prefix_hex="32",
    bip32_prv_prefix_hex="019d9cfe", bip32_pub_prefix_hex="019da462", bech32_hrp="ltc")



from . import errno

from .flags import (
    SEQUENCE_LOCKTIME_DISABLE_FLAG, SEQUENCE_LOCKTIME_TYPE_FLAG,
    VERIFY_DISCOURAGE_UPGRADABLE_NOPS,
    VERIFY_CHECKLOCKTIMEVERIFY,
    VERIFY_MINIMALIF, VERIFY_CHECKSEQUENCEVERIFY,
)

from pycoin.coins.SolutionChecker import ScriptError


def make_bad_opcode(opcode, even_outside_conditional=False, err=errno.BAD_OPCODE):
    def bad_opcode(vm):
        raise ScriptError("invalid opcode %s at %d" % (opcode, vm.pc-1), err)
    bad_opcode.outside_conditional = even_outside_conditional
    return bad_opcode


def do_OP_CODESEPARATOR(vm):
    vm.begin_code_hash = vm.pc


def do_OP_TOALTSTACK(vm):
    vm.altstack.append(vm.pop())


def do_OP_RESERVED(vm):
    if vm.conditional_stack.all_if_true():
        raise ScriptError("OP_RESERVED encountered", errno.BAD_OPCODE)
    vm.op_count -= 1


do_OP_RESERVED.outside_conditional = True


def do_OP_FROMALTSTACK(vm):
    if len(vm.altstack) < 1:
        raise ScriptError("alt stack empty", errno.INVALID_ALTSTACK_OPERATION)
    vm.append(vm.altstack.pop())


def discourage_nops(vm):
    if (vm.flags & VERIFY_DISCOURAGE_UPGRADABLE_NOPS):
        raise ScriptError("discouraging nops", errno.DISCOURAGE_UPGRADABLE_NOPS)


def make_if(reverse_bool=False):
    def f(vm):
        stack = vm.stack
        conditional_stack = vm.conditional_stack
        the_bool = False
        if conditional_stack.all_if_true():
            if len(stack) < 1:
                raise ScriptError("IF with no condition", errno.UNBALANCED_CONDITIONAL)
            item = vm.pop()
            if vm.flags & VERIFY_MINIMALIF:
                if item not in (vm.VM_FALSE, vm.VM_TRUE):
                    raise ScriptError("non-minimal IF", errno.MINIMALIF)
            the_bool = vm.bool_from_script_bytes(item)
        vm.conditional_stack.OP_IF(the_bool, reverse_bool=reverse_bool)
    f.outside_conditional = True
    return f


def do_OP_ELSE(vm):
    vm.conditional_stack.OP_ELSE()


do_OP_ELSE.outside_conditional = True


def do_OP_ENDIF(vm):
    vm.conditional_stack.OP_ENDIF()


do_OP_ENDIF.outside_conditional = True


def do_OP_CHECKLOCKTIMEVERIFY(vm):
    if not (vm.flags & VERIFY_CHECKLOCKTIMEVERIFY):
        if (vm.flags & VERIFY_DISCOURAGE_UPGRADABLE_NOPS):
            raise ScriptError("discouraging nops", errno.DISCOURAGE_UPGRADABLE_NOPS)
        return
    if vm.tx_context.sequence == 0xffffffff:
        raise ScriptError("nSequence equal to 0xffffffff")
    if len(vm.stack) < 1:
        raise ScriptError("empty stack on CHECKLOCKTIMEVERIFY")
    if len(vm.stack[-1]) > 5:
        raise ScriptError("script number overflow")
    max_lock_time = vm.pop_int()
    vm.push_int(max_lock_time)
    if max_lock_time < 0:
        raise ScriptError("top stack item negative on CHECKLOCKTIMEVERIFY")
    era_max = (max_lock_time >= 500000000)
    era_lock_time = (vm.tx_context.lock_time >= 500000000)
    if era_max != era_lock_time:
        raise ScriptError("eras differ in CHECKLOCKTIMEVERIFY")
    if max_lock_time > vm.tx_context.lock_time:
        raise ScriptError("nLockTime too soon")


def _check_sequence_verify(sequence, tx_context_sequence):
    # this mask is applied to extract lock-time from the sequence field
    SEQUENCE_LOCKTIME_MASK = 0xffff

    mask = SEQUENCE_LOCKTIME_TYPE_FLAG | SEQUENCE_LOCKTIME_MASK
    sequence_masked = sequence & mask
    tx_sequence_masked = tx_context_sequence & mask
    if not (((tx_sequence_masked < SEQUENCE_LOCKTIME_TYPE_FLAG) and
             (sequence_masked < SEQUENCE_LOCKTIME_TYPE_FLAG)) or
            ((tx_sequence_masked >= SEQUENCE_LOCKTIME_TYPE_FLAG) and
             (sequence_masked >= SEQUENCE_LOCKTIME_TYPE_FLAG))):
        raise ScriptError("sequence numbers not comparable")
    if sequence_masked > tx_sequence_masked:
        raise ScriptError("sequence number too small")


def do_OP_CHECKSEQUENCEVERIFY(vm):
    if not (vm.flags & VERIFY_CHECKSEQUENCEVERIFY):
        if (vm.flags & VERIFY_DISCOURAGE_UPGRADABLE_NOPS):
            raise ScriptError("discouraging nops", errno.DISCOURAGE_UPGRADABLE_NOPS)
        return
    if len(vm.stack) < 1:
        raise ScriptError("empty stack on CHECKSEQUENCEVERIFY", errno.INVALID_STACK_OPERATION)
    if len(vm.stack[-1]) > 5:
        raise ScriptError("script number overflow", errno.INVALID_STACK_OPERATION+1)
    sequence = vm.pop_int()
    vm.push_int(sequence)
    if sequence < 0:
        raise ScriptError(
            "top stack item negative on CHECKSEQUENCEVERIFY", errno.NEGATIVE_LOCKTIME)
    if sequence & SEQUENCE_LOCKTIME_DISABLE_FLAG:
        return
    # do the actual check
    if vm.tx_context.version < 2:
        raise ScriptError("CHECKSEQUENCEVERIFY: bad tx version", errno.UNSATISFIED_LOCKTIME)
    if vm.tx_context.sequence & SEQUENCE_LOCKTIME_DISABLE_FLAG:
        raise ScriptError("CHECKSEQUENCEVERIFY: locktime disabled")

    _check_sequence_verify(sequence, vm.tx_context.sequence)


def extra_opcodes():
    d = {}
    BAD_OPCODES = "OP_VERIF OP_VERNOTIF ".split()
    for opcode in BAD_OPCODES:
        d[opcode] = make_bad_opcode(opcode, even_outside_conditional=True)

    DISABLED_OPCODES = (
        "OP_CAT OP_SUBSTR OP_LEFT OP_RIGHT OP_INVERT OP_AND OP_OR OP_XOR OP_2MUL OP_2DIV OP_MUL "
        "OP_DIV OP_MOD OP_LSHIFT OP_RSHIFT".split())
    for opcode in DISABLED_OPCODES:
        d[opcode] = make_bad_opcode(
            opcode, even_outside_conditional=True, err=errno.DISABLED_OPCODE)

    BAD_OPCODES_OUTSIDE_IF = "OP_NULLDATA OP_PUBKEYHASH OP_PUBKEY OP_INVALIDOPCODE".split()
    for opcode in BAD_OPCODES_OUTSIDE_IF:
        d[opcode] = make_bad_opcode(opcode, even_outside_conditional=False)

    NOP_SET = (
        "OP_NOP1 OP_NOP3 OP_NOP4 OP_NOP5 OP_NOP6 OP_NOP7 OP_NOP8 OP_NOP9 OP_NOP10".split())
    for opcode in NOP_SET:
        d[opcode] = discourage_nops

    d["OP_IF"] = make_if()
    d["OP_NOTIF"] = make_if(reverse_bool=True)

    for i in (1, 2, 4):
        d["OP_PUSHDATA%d" % i] = lambda s: 0

    for v in range(0, 128):
        d["OP_%d" % v] = lambda s: 0
    return d


"""
The MIT License (MIT)

Copyright (c) 2013-2017 by Richard Kiss

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
"""



from pycoin.networks.bitcoinish import create_bitcoinish_network

network = create_bitcoinish_network(
    symbol="TVI", network_name="Viacoin", subnet_name="testnet",
    wif_prefix_hex="ff", address_prefix_hex="7f", pay_to_script_prefix_hex="c4",
    bip32_prv_prefix_hex="04358394", bip32_pub_prefix_hex="043587cf")



from pycoin.networks.bitcoinish import create_bitcoinish_network

network = create_bitcoinish_network(
    symbol="DFC", network_name="DEFCOIN", subnet_name="mainnet",
    wif_prefix_hex="9e", address_prefix_hex="1e", pay_to_script_prefix_hex="05",
    bip32_prv_prefix_hex="02fa54d7", bip32_pub_prefix_hex="02fa54ad")



from ..encoding.hexbytes import bytes_as_hex
from ..intbytes import indexbytes, int2byte


def make_const_handler(data):
    """
    Create a handler for a data opcode that returns a constant.
    """
    data = bytes_as_hex(data)

    def constant_data_opcode_handler(script, pc, verify_minimal_data=False):
        return pc+1, data
    return constant_data_opcode_handler


def make_sized_handler(size, const_values, non_minimal_data_handler):
    """
    Create a handler for a data opcode that returns literal data of a fixed size.
    """
    const_values = list(const_values)

    def constant_size_opcode_handler(script, pc, verify_minimal_data=False):
        pc += 1
        data = bytes_as_hex(script[pc:pc+size])
        if len(data) < size:
            return pc+1, None
        if verify_minimal_data and data in const_values:
            non_minimal_data_handler("not minimal push of %s" % repr(data))
        return pc+size, data
    return constant_size_opcode_handler


def make_variable_handler(dec_f, sized_values, min_size, non_minimal_data_handler):
    """
    Create a handler for a data opcode that returns literal data of a variable size
    that's fetched and decoded by the function dec_f.
    """
    sized_values = list(sized_values)

    def f(script, pc, verify_minimal_data=False):
        size, pc = dec_f(script, pc)
        data = bytes_as_hex(script[pc:pc+size])
        if len(data) < size:
            return pc+1, None
        if verify_minimal_data:
            if size in sized_values or size <= min_size:
                non_minimal_data_handler("not minimal push of data with size %d" % size)
        return pc+size, data
    return f


def make_sized_encoder(opcode_value):
    """
    Create an encoder that encodes the given opcode value as binary data
    and appends the given data.
    """
    opcode_bin = int2byte(opcode_value)

    def f(data):
        return opcode_bin + data
    return f


class ScriptStreamer(object):
    """
    This class manages encoding and decoding instructions and data from a script.

    Most instructions operate on existing stack data. Then there are some opcodes that
    push constant data onto the stack. These are data opcodes.
    There are three classes of data opcodes:
        * CONSTANT_OPCODE: opcodes that push a constant
        * SIZED_OPCODE: opcodes that push a constant number of subsequent bytes
        * VARIABLE_OPCODE: opcodes that push an encoding of the number of bytes, followed by those bytes

    This class is also aware of "minimal encoding", that is, encoding data using
    the shortest data sequence possible, and can verify that scripts use it.
    This gives a "canonical" version of solution scripts, which can be used to
    help reduce the impact of malleability.
    """
    def __init__(self, opcode_const_list, opcode_sized_list, opcode_variable_list,
                 opcode_lookup, non_minimal_data_handler):
        """
        :param opcode_const_list: list of ("OPCODE_NAME", const_value) pairs, where
            OPCODE_NAME is the name of an opcode and const_value is the value to be pushed
            for that opcode
        :param opcode_sized_list: list of ("OPCODE_NAME", data_size) pairs, where
            OPCODE_NAME is the name of an opcode and data_size is the size
        :param opcode_variable_list: list of ("OPCODE_NAME", max_data_size, enc_f, dec_f)
            tuples where OPCODE_NAME is an opcode, max_data_size is the maximum amount of
            data that can be pushed by this opcode, enc_f is the encoder for the size of the data,
            dec_f is the decoder for the size of the data.
            enc_f should have signature (bin_data) and return an integer (length)
            dec_f should have signature (bin_data) and return offset, data where
                offset is the integer count of bytes consumed and data the decoded result

        :param opcode_lookup: dictionary with entries "OPCODE_NAME" => byte
        :param non_minimal_data_handler: function called when data encoded non-minimally
        """

        # build encoders
        const_pairs = [(opcode_lookup.get(opcode), val) for opcode, val in opcode_const_list]
        self.const_encoder = {v: int2byte(k) for k, v in const_pairs}

        sized_pairs = [(opcode_lookup.get(opcode), size) for opcode, size in opcode_sized_list]
        self.sized_encoder = {v: make_sized_encoder(k) for k, v in sized_pairs}
        opcode_variable_list = sorted(opcode_variable_list, key=lambda o: o[0])
        self.variable_encoder = list(
            (max_size, opcode_lookup.get(opcode), enc_f)
            for opcode, max_size, enc_f, dec_f in opcode_variable_list)

        # build decoder

        self.decoder = {}

        # deal with variable data opcodes

        min_size = 0
        for o, max_size, enc_f, dec_f in opcode_variable_list:
            self.decoder[opcode_lookup.get(o)] = make_variable_handler(
                dec_f, self.sized_encoder.keys(), min_size, non_minimal_data_handler)
            min_size = max_size + 1

        # deal with sized data opcodes

        self.decoder.update(
            {o: make_sized_handler(
                v, self.const_encoder.keys(), non_minimal_data_handler) for o, v in sized_pairs})

        # deal with constant data opcodes

        self.decoder.update({o: make_const_handler(v) for o, v in const_pairs})

        self.data_opcodes = frozenset(self.decoder.keys())

    def get_opcode(self, script, pc, verify_minimal_data=False):
        """
        Step through the script, returning a tuple with the next opcode, the next
        piece of data (if the opcode represents data), the new PC, and a boolean indicated
        valid parsing.
        """
        opcode = indexbytes(script, pc)
        decoder = self.decoder.get(opcode)
        # lambda s, p, verify_minimal_data: (p+1, None))
        if decoder:
            pc, data = decoder(script, pc, verify_minimal_data=verify_minimal_data)
            is_ok = (data is not None)
        else:
            pc += 1
            data = None
            is_ok = True
        return opcode, data, pc, is_ok

    def compile_push_data(self, data):
        # return bytes that causes the given data to be pushed onto the stack
        if data in self.const_encoder:
            return self.const_encoder.get(data)
        size = len(data)
        if size in self.sized_encoder:
            return self.sized_encoder.get(size)(data)
        for max_size, opcode, enc_f in self.variable_encoder:
            if size <= max_size:
                break
        return int2byte(opcode) + enc_f(len(data)) + data



#!/usr/bin/env python

from __future__ import print_function

import argparse
import sys

from pycoin.encoding.exceptions import EncodingError
from pycoin.encoding.sec import public_pair_to_hash160_sec
from pycoin.networks.registry import network_for_netcode, network_codes

from .ku import parse_key


def add_read_msg_arguments(parser, operation):
    group = parser.add_mutually_exclusive_group()
    group.add_argument('-i', "--input", type=argparse.FileType('r'), default=sys.stdin,
                       help='file containing the message to be %s, instead of stdin' % operation)
    group.add_argument('-m', "--message", help='the message to be %s' % operation)


def create_parser():
    codes = network_codes()
    parser = argparse.ArgumentParser(
        description='Create or verify a text signature using bitcoin standards',
        epilog=('Known networks codes:\n  ' +
                ', '.join(['%s (%s)' % (i, network_for_netcode(i).full_name()) for i in codes]))
    )
    parser.add_argument('-n', "--network", help='specify network (default: BTC = Bitcoin)',
                        default='BTC', choices=codes)

    subparsers = parser.add_subparsers(dest="command")

    sign = subparsers.add_parser('sign', help='sign a message with a private key')
    sign.add_argument('WIF', help='the WIF to sign the message with')
    add_read_msg_arguments(sign, "signed")

    verify = subparsers.add_parser('verify')
    verify.add_argument('signature', help='the signature to verify')
    verify.add_argument('address', nargs="?", help='the address to verify against')
    add_read_msg_arguments(verify, "verified")

    return parser


def get_message_hash(args, message_signer):
    message = args.message
    if message is None:
        message = args.input.read()
    return message_signer.hash_for_signing(message)


def msg_sign(args, parser):
    network = network_for_netcode(args.network)
    message_signer = network.msg
    message_hash = get_message_hash(args, message_signer)
    network, key = parse_key(args.WIF, [network])
    is_compressed = key.is_compressed()
    sig = message_signer.signature_for_message_hash(
        key.secret_exponent(), msg_hash=message_hash, is_compressed=is_compressed)
    print(sig)


def msg_verify(args, parser):
    network = network_for_netcode(args.network)
    message_signer = network.msg
    message_hash = get_message_hash(args, message_signer)
    try:
        pair, is_compressed = message_signer.pair_for_message_hash(args.signature, msg_hash=message_hash)
    except EncodingError:
        pass
    ta = network.address.for_p2pkh(public_pair_to_hash160_sec(pair, compressed=is_compressed))
    if args.address:
        if ta == args.address:
            print("signature ok")
            return 0
        else:
            print("bad signature, matches %s" % ta)
            return 1
    else:
        print(ta)


def msg(args, parser):
    command_lookup = {
        "sign": msg_sign,
        "verify": msg_verify
    }
    f = command_lookup.get(args.command)
    if f is None:
        parser.error("no subcommand given: %s" % ' '.join(sorted(command_lookup.keys())))
    f(args, parser)


def main():
    parser = create_parser()
    args = parser.parse_args()
    return msg(args, parser)


if __name__ == '__main__':
    sys.exit(main())



import binascii
import io

from ..intbytes import int2byte

from pycoin.coins.SolutionChecker import ScriptError


class ScriptTools(object):

    def __init__(self, opcode_list, IntStreamer, scriptStreamer):
        self.intStreamer = IntStreamer
        self.scriptStreamer = scriptStreamer

        self.opcode_to_int = dict(o for o in opcode_list)
        self.int_to_opcode = dict(reversed(o) for o in opcode_list)

    def int_for_opcode(self, opcode):
        return self.opcode_to_int.get(opcode)

    def compile_expression(self, t):
        if (t[0], t[-1]) == ('[', ']'):
            return binascii.unhexlify(t[1:-1])
        if t.startswith("'") and t.endswith("'"):
            return t[1:-1].encode("utf8")
        try:
            t0 = int(t)
            if abs(t0) <= 0xffffffffffffffff and t[0] != '0':
                return self.intStreamer.int_to_script_bytes(t0)
        except (SyntaxError, ValueError):
            pass
        try:
            return binascii.unhexlify(t)
        except Exception:
            pass
        raise SyntaxError("unknown expression %s" % t)

    def compile(self, s):
        """
        Compile the given script. Returns a bytes object with the compiled script.
        """
        f = io.BytesIO()
        for t in s.split():
            t_up = t.upper()
            if t_up in self.opcode_to_int:
                f.write(int2byte(self.opcode_to_int[t]))
            elif ("OP_%s" % t_up) in self.opcode_to_int:
                f.write(int2byte(self.opcode_to_int["OP_%s" % t]))
            elif t_up.startswith("0X"):
                d = binascii.unhexlify(t[2:])
                f.write(d)
            else:
                v = self.compile_expression(t)
                self.write_push_data([v], f)
        return f.getvalue()

    def disassemble_for_opcode_data(self, opcode, data):
        # TODO: check data for int or string representation
        opcode_str = self.int_to_opcode.get(opcode, "???")
        if data is not None and len(data) > 0 and opcode_str.startswith("OP_PUSH"):
            return "[%s]" % binascii.hexlify(data).decode("utf8")
        return opcode_str

    def get_opcodes(self, script, verify_minimal_data=False, pc=0):
        """
        Iterator. Return opcode, data, pc, new_pc at each step
        """
        while pc < len(script):
            opcode, data, new_pc, is_ok = self.scriptStreamer.get_opcode(
                script, pc, verify_minimal_data=verify_minimal_data)
            yield opcode, data, pc, new_pc
            pc = new_pc

    def opcode_list(self, script):
        """Disassemble the given script. Returns a list of opcodes."""
        opcodes = []
        new_pc = 0
        try:
            for opcode, data, pc, new_pc in self.get_opcodes(script):
                opcodes.append(self.disassemble_for_opcode_data(opcode, data))
        except ScriptError:
            opcodes.append(binascii.hexlify(script[new_pc:]).decode("utf8"))

        return opcodes

    def disassemble(self, script):
        """Disassemble the given script. Returns a string."""
        return ' '.join(self.opcode_list(script))

    def write_push_data(self, data_list, f):
        # return bytes that causes the given data to be pushed onto the stack
        for t in data_list:
            f.write(self.scriptStreamer.compile_push_data(t))

    def compile_push_data_list(self, data_list):
        return b''.join(self.scriptStreamer.compile_push_data(d) for d in data_list if d is not None)



import struct


def parse_satoshi_int(f, v=None):
    if v is None:
        v = ord(f.read(1))
    if v == 253:
        v = struct.unpack("<H", f.read(2))[0]
    elif v == 254:
        v = struct.unpack("<L", f.read(4))[0]
    elif v == 255:
        v = struct.unpack("<Q", f.read(8))[0]
    return v


def stream_satoshi_int(f, v):
    if v < 253:
        f.write(struct.pack("<B", v))
    elif v <= 65535:
        f.write(b'\xfd' + struct.pack("<H", v))
    elif v <= 0xffffffff:
        f.write(b'\xfe' + struct.pack("<L", v))
    else:
        f.write(b'\xff' + struct.pack("<Q", v))



import hmac
import hashlib
import struct

from pycoin.encoding.bytes32 import from_bytes_32
from pycoin.encoding.sec import public_pair_to_sec


def ascend_bip32(bip32_pub_node, secret_exponent, child):
    """
    Given a BIP32Node with public derivation child "child" with a known private key,
    return the secret exponent for the bip32_pub_node.
    """
    i_as_bytes = struct.pack(">l", child)
    sec = public_pair_to_sec(bip32_pub_node.public_pair(), compressed=True)
    data = sec + i_as_bytes
    I64 = hmac.HMAC(key=bip32_pub_node._chain_code, msg=data, digestmod=hashlib.sha512).digest()
    I_left_as_exponent = from_bytes_32(I64[:32])
    return (secret_exponent - I_left_as_exponent) % bip32_pub_node._generator.order()


def crack_bip32(bip32_pub_node, secret_exponent, path):
    paths = path.split("/")
    while len(paths):
        path = int(paths.pop())
        secret_exponent = ascend_bip32(bip32_pub_node.subkey_for_path("/".join(paths)), secret_exponent, path)
    return bip32_pub_node.__class__(
        bip32_pub_node._chain_code, bip32_pub_node._depth, bip32_pub_node._parent_fingerprint,
        bip32_pub_node._child_index, secret_exponent=secret_exponent)



def generate_shared_public_key(my_private_key, their_public_pair, generator):
    """
    Two parties each generate a private key and share their public key with the
    other party over an insecure channel. The shared public key can be generated by
    either side, but not by eavesdroppers. You can then use the entropy from the
    shared public key to created a common symmetric key for encryption. (This
    is beyond of the scope of pycoin.)

    See also <https://en.wikipedia.org/wiki/Key_exchange>

    :param my_private_key: an integer private key
    :param their_public_pair: a pair ``(x, y)`` representing a public key for the ``generator``
    :param generator: a :class:`Generator <pycoin.ecdsa.Generator.Generator>`
    :returns: a :class:`Point <pycoin.ecdsa.Point.Point>`, which can be used as a shared
        public key.
    """
    p = generator.Point(*their_public_pair)
    return my_private_key * p



from ..convention import tx_fee


class SecretExponentMissing(Exception):
    pass


def create_tx(network, spendables, payables, fee="standard", lock_time=0, version=1):
    """
    This function provides the easiest way to create an unsigned transaction.

    All coin values are in satoshis.

    :param spendables: a list of Spendable objects, which act as inputs.
        Each item in the list can be a Spendable, or text from Spendable.as_text,
        or a dictionary from Spendable.as_dict (which might be easier for
        airgapped transactions, for example).
    :param payables: a list where each entry is a address, or a tuple of
        (address, coin_value). If the coin_value is missing or
        zero, this address is thrown into the "split pool". Funds not
        explicitly claimed by the fee or an address are shared as
        equally as possible among the split pool. All coins are consumed:
        if the amount to be split does not divide evenly, some of the earlier
        addresses will get an extra satoshi.
    :param fee: an integer, or the (deprecated) string "standard" for it to be calculated
    :param version: (optional) the version to use in the transaction. Defaults to 1.
    :param lock_time: (optional) the lock_time to use in the transaction. Defaults to 0.
    :return: :class:`Tx <Tx>` object, with unspents populated
    :rtype: pycoin.tx.Tx.Tx

    Usage::

        >>> spendables = spendables_for_address("1BgGZ9tcN4rm9KBzDn7KprQz87SZ26SAMH")
        >>> tx = create_tx(network, spendables, ["1cMh228HTCiwS8ZsaakH8A8wze1JR5ZsP"], fee=0)

    This will move all available reported funds from 1BgGZ9tcN4rm9KBzDn7KprQz87SZ26SAMH
    to 1cMh228HTCiwS8ZsaakH8A8wze1JR5ZsP, with no transaction fees (which means it might
    take a while to confirm, possibly never).
    """

    Tx = network.tx

    def _fix_spendable(s):
        if isinstance(s, Tx.Spendable):
            return s
        if not hasattr(s, "keys"):
            return Tx.Spendable.from_text(s)
        return Tx.Spendable.from_dict(s)

    spendables = [_fix_spendable(s) for s in spendables]
    txs_in = [spendable.tx_in() for spendable in spendables]

    txs_out = []
    for payable in payables:
        if len(payable) == 2:
            address, coin_value = payable
        else:
            address = payable
            coin_value = 0
        script = network.contract.for_address(address)
        txs_out.append(Tx.TxOut(coin_value, script))

    tx = Tx(version=version, txs_in=txs_in, txs_out=txs_out, lock_time=lock_time)
    tx.set_unspents(spendables)

    distribute_from_split_pool(tx, fee)
    return tx


def split_with_remainder(total_amount, split_count):
    value_each, extra_count = divmod(total_amount, split_count)
    for _ in range(extra_count):
        yield value_each + 1
    for _ in range(split_count-extra_count):
        yield value_each


def distribute_from_split_pool(tx, fee):
    """
    :param tx: a transaction
    :param fee: integer, satoshi value to set aside for transaction fee

    This function looks at TxOut items of a transaction tx and
    and puts TxOut items with a coin value of zero into a "split pool".
    Funds not explicitly claimed by the fee or other TxOut items are
    shared as equally as possible among the split pool. If the amount
    to be split does not divide evenly, some of the earlier TxOut items
    will get an extra satoshi. This function modifies `tx` in place.
    """

    # calculate fees
    if fee == 'standard':
        # TODO: improve this
        # 1: the tx is not fully built out, so it will actually be larger than implied at this point
        # 2: recommended_fee_for_tx gives estimates that are too high
        fee = tx_fee.recommended_fee_for_tx(tx)

    zero_txs_out = [tx_out for tx_out in tx.txs_out if tx_out.coin_value == 0]
    zero_count = len(zero_txs_out)
    if zero_count > 0:
        total_coin_value = sum(spendable.coin_value for spendable in tx.unspents)
        coins_allocated = sum(tx_out.coin_value for tx_out in tx.txs_out) + fee
        remaining_coins = total_coin_value - coins_allocated
        if remaining_coins < 0:
            raise ValueError("insufficient inputs for outputs")
        if remaining_coins < zero_count:
            raise ValueError("not enough to pay nonzero amounts to at least one of the unspecified outputs")
        for value, tx_out in zip(split_with_remainder(remaining_coins, zero_count), zero_txs_out):
            tx_out.coin_value = value
    return zero_count


def sign_tx(network, tx, wifs=[], **kwargs):
    """
    :param tx: a transaction
    :param wifs: the list of WIFs required to sign this transaction.
    :return: :class:`Tx <Tx>` object, modified in place

    This is a convenience function used to sign a transaction.
    The transaction must have "unspents" set by, for example, calling tx.unspents_from_db.

    Returns the signed Tx transaction, or raises an exception.

    Usage::

        >> sign_tx(network, tx, wifs=["KwDiBf89QgGbjEhKnhXJuH7LrciVrZi3qYjgd9M7rFU73sVHnoWn"])
    """
    keychain = network.keychain()
    keychain.add_secrets((network.parse.wif(_) for _ in wifs))
    solver = tx.Solver(tx)
    solver.sign(keychain, **kwargs)


def create_signed_tx(network, spendables, payables, wifs=[], fee="standard",
                     lock_time=0, version=1, **kwargs):
    """
    This convenience function calls :func:`create_tx` and :func:`sign_tx` in turn. Read the documentation
    for those functions for information on the parameters.

    Usage::

        >>> spendables = spendables_for_address("1BgGZ9tcN4rm9KBzDn7KprQz87SZ26SAMH")
        >>> wifs = ["KwDiBf89QgGbjEhKnhXJuH7LrciVrZi3qYjgd9M7rFU73sVHnoWn"]
        >>> payables = ["1cMh228HTCiwS8ZsaakH8A8wze1JR5ZsP"]
        >>> tx = create_signed_tx(network, spendables, payables, wifs=wifs, fee=0)

    This will move all available reported funds from 1BgGZ9tcN4rm9KBzDn7KprQz87SZ26SAMH
    to 1cMh228HTCiwS8ZsaakH8A8wze1JR5ZsP, with no transaction fees (which means it might
    take a while to confirm, possibly never).
    """

    tx = create_tx(network, spendables, payables, fee=fee, lock_time=lock_time, version=version)
    sign_tx(network, tx, wifs=wifs, **kwargs)
    for idx, tx_out in enumerate(tx.txs_in):
        if not tx.is_solution_ok(idx):
            raise SecretExponentMissing("failed to sign spendable for %s" %
                                        tx.unspents[idx])
    return tx



import io
import unittest

from pycoin.encoding.hexbytes import h2b
from pycoin.symbols.btc import network


Tx = network.tx


class PayToTest(unittest.TestCase):

    def test_recognize_multisig(self):
        h = (
            "010000000139c92b102879eb95f14e7344e4dd7d481e1238b1bfb1fa0f735068d2927b"
            "231400000000910047304402208fc06d216ebb4b6a3a3e0f906e1512c372fa8a9c2a92"
            "505d04e9b451ea7acd0c0220764303bb7e514ddd77855949d941c934e9cbda8e3c3827"
            "bfdb5777477e73885b014730440220569ec6d2e81625dd18c73920e0079cdb4c1d67d3"
            "d7616759eb0c18cf566b3d3402201c60318f0a62e3ba85ca0f158d4dfe63c0779269eb"
            "6765b6fc939fc51e7a8ea901ffffffff0140787d01000000001976a914641ad5051edd"
            "97029a003fe9efb29359fcee409d88ac0000000040787d0100000000c952410496ec45"
            "f878b62c46c4be8e336dff7cc58df9b502178cc240eb3d31b1266f69f5767071aa3e01"
            "7d1b82a0bb28dab5e27d4d8e9725b3e68ed5f8a2d45c730621e34104cc71eb30d653c0"
            "c3163990c47b976f3fb3f37cccdcbedb169a1dfef58bbfbfaff7d8a473e7e2e6d317b8"
            "7bafe8bde97e3cf8f065dec022b51d11fcdd0d348ac4410461cbdcc5409fb4b4d42b51"
            "d33381354d80e550078cb532a34bfa2fcfdeb7d76519aecc62770f5b0e4ef8551946d8"
            "a540911abe3e7854a26f39f58b25c15342af53ae")
        f = io.BytesIO(h2b(h))
        tx = Tx.parse(f)
        tx.parse_unspents(f)
        self.assertEqual(tx.id(), "10c61e258e0a2b19b245a96a2d0a1538fe81cd4ecd547e0a3df7ed6fd3761ada")
        script = tx.unspents[0].script
        multisig_info = network.contract.info_for_script(script)
        del multisig_info["type"]
        s = network.contract.for_multisig(**multisig_info)
        self.assertEqual(s, script)

    def test_nulldata(self):
        OP_RETURN = network.script.compile("OP_RETURN")
        for sample in [b'test', b'me', b'a', b'39qEwuwyb2cAX38MFtrNzvq3KV9hSNov3q', b'', b'0'*80]:
            sample_script = OP_RETURN + sample
            sc = network.contract.for_nulldata(sample)
            info = network.contract.info_for_script(sc)
            self.assertEqual(info.get("data"), sample)
            self.assertEqual(sc, sample_script)
            out = Tx.TxOut(1, sc)
            # ensure we can create a tx
            Tx(0, [], [out])
            # convert between asm and back to ensure no bugs with compilation
            # BRAIN DAMAGE: this doesn't work yet
            # self.assertEqual(sc, network.script.compile(network.script.disassemble(sc)))

    def test_nulldata_push(self):
        OP_RETURN = network.script.compile("OP_RETURN")
        for sample in [b'test', b'me', b'a', b'39qEwuwyb2cAX38MFtrNzvq3KV9hSNov3q', b'', b'0'*80]:
            sample_push = network.script.compile_push_data_list([sample])
            sample_script = OP_RETURN + sample_push
            sc = network.contract.for_nulldata_push(sample)
            info = network.contract.info_for_script(sc)
            self.assertEqual(info.get("data"), sample_push)
            self.assertEqual(sc, sample_script)
            out = Tx.TxOut(1, sc)
            # ensure we can create a tx
            Tx(0, [], [out])
            # convert between asm and back to ensure no bugs with compilation
            self.assertEqual(sc, network.script.compile(network.script.disassemble(sc)))


if __name__ == "__main__":
    unittest.main()



import hashlib

from pycoin.encoding.hash import hash160
from pycoin.encoding.sec import public_pair_to_hash160_sec


def build_hash160_lookup(secret_exponents, generators):
    d = {}
    for secret_exponent in secret_exponents:
        for generator in generators:
            public_pair = secret_exponent * generator
            for compressed in (True, False):
                hash160 = public_pair_to_hash160_sec(public_pair, compressed=compressed)
                d[hash160] = (secret_exponent, public_pair, compressed, generator)
    return d


def build_p2sh_lookup(scripts):
    d1 = dict((hash160(s), s) for s in scripts)
    d1.update((hashlib.sha256(s).digest(), s) for s in scripts)
    return d1


def build_sec_lookup(sec_values):
    d = {}
    for sec in sec_values or []:
        d[hash160(sec)] = sec
    return d



from pycoin.networks.bitcoinish import create_bitcoinish_network

network = create_bitcoinish_network(
    symbol="MEC", network_name="Megacoin", subnet_name="mainnet",
    wif_prefix_hex="B2", sec_prefix="MECSEC:", address_prefix_hex="32", pay_to_script_prefix_hex="05",
    bip32_prv_prefix_hex="0488ADE4", bip32_pub_prefix_hex="0488B21E",
    magic_header_hex="EDE0E4EE", default_port=7951,
    dns_bootstrap=[]
)



#!/usr/bin/env python

import argparse
from pycoin.encoding.hexbytes import b2h
from pycoin.networks.registry import network_codes, network_for_netcode
from pycoin.networks.default import get_current_netcode


def create_parser():
    codes = network_codes()
    EPILOG = ('Known networks codes:\n  ' +
              ', '.join(['%s (%s)' % (i, network_for_netcode(i).full_name()) for i in codes]))

    parser = argparse.ArgumentParser(
        description="Compile or disassemble scripts.",
        epilog=EPILOG)

    parser.add_argument('-n', "--network", default=get_current_netcode(), choices=codes,
                        help=('Network code (environment variable PYCOIN_DEFAULT_NETCODE '
                              'or "BTC"=Bitcoin mainnet if unset)'))

    parser.add_argument("argument", nargs="+", help='script to compile. To dump hex, prefix with 0x')

    return parser


def coinc(args, parser):
    network = network_for_netcode(args.network)

    for arg in args.argument:
        info = info_for_arg(arg, network)
        for k in ("compiled_script_hex address_p2s preimage_p2s_hex "
                  "address_p2s_wit underlying_script disassembled_script").split():
            print(info[k])


def info_for_arg(arg, network):
    d = {}
    compiled_script = network.script.compile(arg)
    d["compiled_script_hex"] = "0x%s" % b2h(compiled_script)

    address_p2s = network.address.for_p2s(compiled_script)
    d["address_p2s"] = address_p2s
    d["preimage_p2s_hex"] = b2h(network.contract.for_address(address_p2s))

    address_p2s_wit = network.address.for_p2s_wit(compiled_script)
    d["address_p2s_wit"] = address_p2s_wit
    d["underlying_script"] = b2h(network.contract.for_address(address_p2s_wit))

    d["disassembled_script"] = network.script.disassemble(compiled_script)
    return d


def main():
    parser = create_parser()
    args = parser.parse_args()
    coinc(args, parser)


if __name__ == '__main__':
    main()



from pycoin.networks.bitcoinish import create_bitcoinish_network

network = create_bitcoinish_network(
    symbol="DCR", network_name="Decred", subnet_name="mainnet",
    wif_prefix_hex="22de", address_prefix_hex="073f", pay_to_script_prefix_hex="071a",
    bip32_prv_prefix_hex="02fda4e8", bip32_pub_prefix_hex="02fda926")




from pycoin.coins.SolutionChecker import ScriptError
from pycoin.ecdsa.secp256k1 import secp256k1_generator
from pycoin.satoshi import errno, opcodes
from pycoin.satoshi.IntStreamer import IntStreamer
from pycoin.satoshi.flags import VERIFY_MINIMALDATA

from .ScriptStreamer import BitcoinScriptStreamer

from pycoin.vm.VM import VM


from .make_instruction_lookup import make_instruction_lookup


class BitcoinVM(VM):
    IntStreamer = IntStreamer

    VM_FALSE = IntStreamer.int_to_script_bytes(0)
    VM_TRUE = IntStreamer.int_to_script_bytes(1)

    INSTRUCTION_LOOKUP = make_instruction_lookup(opcodes.OPCODE_LIST)
    ScriptStreamer = BitcoinScriptStreamer

    def pop_int(self):
        return self.IntStreamer.int_from_script_bytes(self.pop(), require_minimal=self.flags & VERIFY_MINIMALDATA)

    def pop_nonnegative(self):
        v = self.pop_int()
        if v < 0:
            raise ScriptError("unexpectedly got negative value", errno.INVALID_STACK_OPERATION)
        return v

    def push_int(self, v):
        self.append(self.IntStreamer.int_to_script_bytes(v))

    @classmethod
    def bool_from_script_bytes(class_, v, require_minimal=False):
        v = class_.IntStreamer.int_from_script_bytes(v, require_minimal=require_minimal)
        if require_minimal:
            if v not in (class_.VM_FALSE, class_.VM_TRUE):
                raise ScriptError("non-minimally encoded", errno.UNKNOWN_ERROR)
        return bool(v)

    @classmethod
    def bool_to_script_bytes(class_, v):
        return class_.VM_TRUE if v else class_.VM_FALSE

    @classmethod
    def generator_for_signature_type(class_, signature_type):
        return secp256k1_generator



import hashlib
import hmac
import logging
import struct

from ..encoding.bytes32 import from_bytes_32, to_bytes_32
from ..encoding.sec import public_pair_to_sec

logger = logging.getLogger(__name__)


_SUBKEY_VALIDATION_LOG_ERR_FMT = """
BUY A LOTTO TICKET RIGHT NOW! (And consider giving up your wallet to
science!)

You have stumbled across an astronomically unlikely scenario. Your HD
wallet contains an invalid subkey. Having access to this information would
be incredibly valuable to the Bitcoin development community.

If you are inclined to help, please make sure to back up this wallet (or
any outputted information) onto a USB drive and e-mail "Richard Kiss"
<him@richardkiss.com> or "Matt Bogosian" <mtb19@columbia.edu> for
instructions on how best to donate it without losing your bitcoins.

WARNING: DO NOT SEND ANY WALLET INFORMATION UNLESS YOU WANT TO LOSE ALL
THE BITCOINS IT CONTAINS.
""".strip()


class DerivationError(ValueError):
    pass


def subkey_secret_exponent_chain_code_pair(
        generator, secret_exponent, chain_code_bytes, i, is_hardened, public_pair=None):
    """
    Yield info for a child node for this node.

    generator:
        the ecdsa generator
    secret_exponent:
        base secret exponent
    chain_code:
        base chain code
    i:
        the index for this node.
    is_hardened:
        use "hardened key derivation". The public version of this node cannot calculate this child.
    public_pair:
        the public_pair for the given secret exponent. If you leave it None, it's calculated for you
        (but then it's slower)

    Returns a pair (new_secret_exponent, new_chain_code)
    """
    ORDER = generator.order()
    i_as_bytes = struct.pack(">L", i)

    if is_hardened:
        data = b'\0' + to_bytes_32(secret_exponent) + i_as_bytes
    else:
        if public_pair is None:
            public_pair = secret_exponent * generator
        sec = public_pair_to_sec(public_pair, compressed=True)
        data = sec + i_as_bytes

    I64 = hmac.HMAC(key=chain_code_bytes, msg=data, digestmod=hashlib.sha512).digest()
    I_left_as_exponent = from_bytes_32(I64[:32]) % ORDER
    new_secret_exponent = (I_left_as_exponent + secret_exponent) % ORDER
    if new_secret_exponent == 0:
        logger.critical(_SUBKEY_VALIDATION_LOG_ERR_FMT)
        raise DerivationError('k_{} == 0'.format(i))

    new_chain_code = I64[32:]
    return new_secret_exponent, new_chain_code


def subkey_public_pair_chain_code_pair(generator, public_pair, chain_code_bytes, i):
    """
    Yield info for a child node for this node.

    generator:
        the ecdsa generator
    public_pair:
        base public pair
    chain_code:
        base chain code
    i:
        the index for this node.

    Returns a pair (new_public_pair, new_chain_code)
    """
    INFINITY = generator.infinity()
    ORDER = generator.order()
    i_as_bytes = struct.pack(">l", i)
    sec = public_pair_to_sec(public_pair, compressed=True)
    data = sec + i_as_bytes

    I64 = hmac.HMAC(key=chain_code_bytes, msg=data, digestmod=hashlib.sha512).digest()
    I_left_as_exponent = from_bytes_32(I64[:32]) % ORDER
    the_point = I_left_as_exponent * generator + generator.Point(*public_pair)
    if the_point == INFINITY:
        logger.critical(_SUBKEY_VALIDATION_LOG_ERR_FMT)
        raise DerivationError('K_{} == {}'.format(i, the_point))

    new_chain_code = I64[32:]
    return the_point, new_chain_code


"""
A BIP0032-style hierarchical wallet.

Implement a BIP0032-style hierarchical wallet which can create public
or private wallet keys. Each key can create many child nodes. Each node
has a wallet key and a corresponding private & public key, which can
be used to generate Bitcoin addresses or WIF private keys.

At any stage, the private information can be stripped away, after which
descendants can only produce public keys.

Private keys can also generate "hardened" children, which cannot be
generated by the corresponding public keys. This is useful for generating
"change" addresses, for example, which there is no need to share with people
you give public keys to.


The MIT License (MIT)

Copyright (c) 2013 by Richard Kiss

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
"""




class ConditionalStack(object):
    def __init__(self, error_f):
        self.true_count = 0
        self.false_count = 0
        self.error_f = error_f

    def all_if_true(self):
        return self.false_count == 0

    def OP_IF(self, the_bool, reverse_bool=False):
        if self.false_count > 0:
            self.false_count += 1
            return
        if reverse_bool:
            the_bool = not the_bool
        if the_bool:
            self.true_count += 1
        else:
            self.false_count = 1

    def OP_ELSE(self):
        if self.false_count > 1:
            return
        if self.false_count == 1:
            self.false_count = 0
            self.true_count += 1
        else:
            if self.true_count == 0:
                return self.error_f("OP_ELSE without OP_IF")
            self.true_count -= 1
            self.false_count += 1

    def OP_ENDIF(self):
        if self.false_count > 0:
            self.false_count -= 1
        else:
            if self.true_count == 0:
                return self.error_f("OP_ENDIF without OP_IF")
            self.true_count -= 1

    def check_final_state(self):
        if self.false_count > 0 or self.true_count > 0:
            return self.error_f("missing ENDIF")

    def __repr__(self):
        if self.true_count or self.false_count:
            return "[IfStack true:%d/false:%d]" % (self.true_count, self.false_count)
        return "[]"



from pycoin.networks.bitcoinish import create_bitcoinish_network

network = create_bitcoinish_network(
    symbol="BSD", network_name="BitSend", subnet_name="mainnet",
    wif_prefix_hex="CC", sec_prefix="BSDSEC:", address_prefix_hex="66", pay_to_script_prefix_hex="05",
    bip32_prv_prefix_hex="0488ADE4", bip32_pub_prefix_hex="0488B21E",
    magic_header_hex="A3D5C2F9", default_port=8886,
    dns_bootstrap=[
        "seed.mybitsend.com"
    ])



from pycoin.networks.bitcoinish import create_bitcoinish_network

network = create_bitcoinish_network(
    symbol="DOGE", network_name="Dogecoin", subnet_name="mainnet",
    wif_prefix_hex="9e", address_prefix_hex="1e", pay_to_script_prefix_hex="16",
    bip32_prv_prefix_hex="02fac398", bip32_pub_prefix_hex="02facafd")



import textwrap
import unittest

from pycoin.symbols.btc import network as BTC
from pycoin.symbols.xtn import network as XTN


def test_against_myself():
    """
    Test code that verifies against ourselves only. Useful but not so great.
    """

    for wif, right_addr in [
                    ('L4gXBvYrXHo59HLeyem94D9yLpRkURCHmCwQtPuWW9m6o1X8p8sp',
                     '1LsPb3D1o1Z7CzEt1kv5QVxErfqzXxaZXv'),
                    ('5KYZdUEo39z3FPrtuX2QbbwGnNP5zTd7yyr2SC1j299sBCnWjss',
                     '1HZwkjkeaoZfTSaJxDw6aKkxp45agDiEzN'),
                ]:
        k = BTC.parse.wif(wif)
        assert k.address() == right_addr

        vk2 = BTC.parse.address(right_addr)
        assert vk2.address() == right_addr

        for i in range(1, 30, 10):
            msg = 'test message %s' % ('A'*i)
            sig = BTC.msg.sign(k, msg, verbose=1)
            assert right_addr in sig

            # check parsing works
            m, a, s = BTC.msg.parse_signed(sig)
            assert m == msg, m
            assert a == right_addr, a

            sig2 = BTC.msg.sign(k, msg, verbose=0)
            assert sig2 in sig, (sig, sig2)

            assert s == sig2, s

            ok = BTC.msg.verify(k, sig2, msg)
            assert ok

            ok = BTC.msg.verify(k, sig2.encode('ascii'), msg)
            assert ok


def test_msg_parse():
    """
        Test against real-world signatures found in the wild.
    """

    # Output from brainwallet in "multibit" mode.
    multibit = '''

-----BEGIN BITCOIN SIGNED MESSAGE-----
This is an example of a signed message.
-----BEGIN BITCOIN SIGNATURE-----
Version: Bitcoin-qt (1.0)
Address: 1HZwkjkeaoZfTSaJxDw6aKkxp45agDiEzN

HCT1esk/TWlF/o9UNzLDANqsPXntkMErf7erIrjH5IBOZP98cNcmWmnW0GpSAi3wbr6CwpUAN4ctNn1T71UBwSc=
-----END BITCOIN SIGNATURE-----

'''
    m, a, s = BTC.msg.parse_signed(multibit)
    assert m == 'This is an example of a signed message.'
    assert a == '1HZwkjkeaoZfTSaJxDw6aKkxp45agDiEzN'
    assert s == ('HCT1esk/TWlF/o9UNzLDANqsPXntkMErf7erIrjH5IBOZ'
                 'P98cNcmWmnW0GpSAi3wbr6CwpUAN4ctNn1T71UBwSc=')
    ok = BTC.msg.verify(a, s, m)
    assert ok

    # Sampled from: https://www.bitrated.com/u/Bit2c.txt on Sep 3/2014
    bit2c = textwrap.dedent(
        '''
        Username: Bit2c
        Public key: 0396267072e597ad5d043db7c73e13af84a77a7212871f1aade607fb0f2f96e1a8
        Public key address: 15etuU8kwLFCBbCNRsgQTvWgrGWY9829ej
        URL: https://www.bitrated.com/u/Bit2c

        -----BEGIN BITCOIN SIGNED MESSAGE-----
        ''' +
        'We will try to contact both parties to gather information and evidence, and do my '
        'best to make rightful judgement. Evidence may be submitted to us on'
        ' https://www.bit2c.co.il/home/contact or in a private message to info@bit2c.co.il'
        ' or in any agreed way.' + '''

        https://www.bit2c.co.il
        -----BEGIN SIGNATURE-----
        15etuU8kwLFCBbCNRsgQTvWgrGWY9829ej
        H2utKkquLbyEJamGwUfS9J0kKT4uuMTEr2WX2dPU9YImg4LeRpyjBelrqEqfM4QC8pJ+hVlQgZI5IPpLyRNxvK8=
        -----END BITCOIN SIGNED MESSAGE-----
        ''')
    m, a, s = BTC.msg.parse_signed(bit2c)
    assert a == '15etuU8kwLFCBbCNRsgQTvWgrGWY9829ej'
    assert s == ('H2utKkquLbyEJamGwUfS9J0kKT4uuMTEr2WX2dPU9YI'
                 'mg4LeRpyjBelrqEqfM4QC8pJ+hVlQgZI5IPpLyRNxvK8=')
    ok = BTC.msg.verify(a, s, m)
    assert ok

    # testnet example
    # Sampled from: http://testnet.bitrated.com/u/bearbin.txt on Sep 3/2014
    # NOTE: Testnet3
    bearbin = textwrap.dedent('''\
        Username: bearbin
        Public key: 03fc594c16779054fc5e119c309215c1f40f2ce104b0169cddeb6d20445bd28f67
        Public key address: n2D9XsQX1mDpFGgYqsfmePTy61LJFQnXQM
        URL: http://testnet.bitrated.com/u/bearbin

        -----BEGIN BITCOIN SIGNED MESSAGE-----
        Contact
        -----------

        bearbin@gmail.com - Email or hangouts (text only).

        /u/bearbin on reddit (slow response, not preferred for use with the service, just for contact).

        Resolution Guidelines:
        -----------------------------

         * Evidence is needed. (e.g. pictures w/ proof that it's you).
         * If anybody fails to respond, money goes to the other person after 2 weeks.
         * Additional terms available on request.

        Pricing
        ----------

         * 0.7% Min 0.003 Max 0.15
         * Payment in advance.
        -----BEGIN SIGNATURE-----
        n2D9XsQX1mDpFGgYqsfmePTy61LJFQnXQM
        IEackZgifpBJs3SqQQ6leUwzvakTZgUKTDuCCn6rVMOQgHlIEzWSYZGQu2H+1chvu68uutzt04cGmsHy/kRIaEc=
        -----END BITCOIN SIGNED MESSAGE-----
        ''')
    m, a, s = XTN.msg.parse_signed(bearbin)
    assert a == 'n2D9XsQX1mDpFGgYqsfmePTy61LJFQnXQM'
    assert s == ('IEackZgifpBJs3SqQQ6leUwzvakTZgUKTDuCCn6rVMOQgH'
                 'lIEzWSYZGQu2H+1chvu68uutzt04cGmsHy/kRIaEc=')
    assert ok


if __name__ == "__main__":
    unittest.main()



from pycoin.block import Block
from pycoin.coins.bitcoin.ScriptTools import BitcoinScriptTools
from pycoin.coins.bitcoin.Tx import Tx
from pycoin.coins.exceptions import ValidationFailureError
from pycoin.coins.tx_utils import create_tx, split_with_remainder, distribute_from_split_pool, sign_tx, create_signed_tx
from pycoin.coins.SolutionChecker import ScriptError
from pycoin.contrib.msg_signing import MessageSigner
from pycoin.contrib.who_signed import WhoSigned
from pycoin.ecdsa.secp256k1 import secp256k1_generator
from pycoin.encoding.b58 import b2a_hashed_base58
from pycoin.key.Keychain import Keychain
from pycoin.key.Key import Key, InvalidSecretExponentError, InvalidPublicPairError
from pycoin.key.BIP32Node import BIP32Node
from pycoin.key.electrum import ElectrumWallet
from pycoin.message.make_parser_and_packer import (
    make_parser_and_packer, standard_messages,
    standard_message_post_unpacks, standard_streamer, standard_parsing_functions
)
from pycoin.encoding.hexbytes import b2h, h2b
from pycoin.satoshi import errno, flags
from pycoin.solve.utils import build_hash160_lookup, build_p2sh_lookup, build_sec_lookup
from pycoin.vm.annotate import Annotate

from .AddressAPI import make_address_api
from .ParseAPI import ParseAPI
from .ContractAPI import ContractAPI
from .parseable_str import parseable_str


class API(object):
    pass


class Network(object):
    def __init__(self, symbol, network_name, subnet_name):
        self.symbol = symbol
        self.network_name = network_name
        self.subnet_name = subnet_name

    def full_name(self):
        return "%s %s" % (self.network_name, self.subnet_name)

    def __repr__(self):
        return "<Network %s>" % self.full_name()


def make_output_for_hwif(network):
    def f(key_data, network, subkey_path, add_output):
        key = None
        if len(key_data) == 74:
            key = network.keys.bip32_deserialize(b'0000' + key_data)
        if key is None:
            return

        yield ("wallet_key", key.hwif(as_private=key.is_private()), None)
        if key.is_private():
            yield ("public_version", key.hwif(as_private=False), None)

        child_number = key.child_index()
        if child_number >= 0x80000000:
            wc = child_number - 0x80000000
            child_index = "%dH (%d)" % (wc, child_number)
        else:
            child_index = "%d" % child_number
        yield ("tree_depth", "%d" % key.tree_depth(), None)
        yield ("fingerprint", b2h(key.fingerprint()), None)
        yield ("parent_fingerprint", b2h(key.parent_fingerprint()), "parent f'print")
        yield ("child_index", child_index, None)
        yield ("chain_code", b2h(key.chain_code()), None)

        yield ("private_key", "yes" if key.is_private() else "no", None)
    return f


def make_output_for_secret_exponent(Key):
    def f(secret_exponent):
        yield ("secret_exponent", '%d' % secret_exponent, None)
        yield ("secret_exponent_hex", '%x' % secret_exponent, " hex")
        key = Key(secret_exponent)
        yield ("wif", key.wif(is_compressed=True), None)
        yield ("wif_uncompressed", key.wif(is_compressed=False), " uncompressed")
    return f


def make_output_for_public_pair(Key, network):
    def f(public_pair):
        yield ("public_pair_x", '%d' % public_pair[0], None)
        yield ("public_pair_y", '%d' % public_pair[1], None)
        yield ("public_pair_x_hex", '%x' % public_pair[0], " x as hex")
        yield ("public_pair_y_hex", '%x' % public_pair[1], " y as hex")
        yield ("y_parity", "odd" if (public_pair[1] & 1) else "even", None)

        key = Key(public_pair=public_pair)
        yield ("key_pair_as_sec", b2h(key.sec(is_compressed=True)), None)
        yield ("key_pair_as_sec_uncompressed", b2h(key.sec(is_compressed=False)), " uncompressed")

        network_name = network.network_name
        hash160_u = key.hash160(is_compressed=False)
        hash160_c = key.hash160(is_compressed=True)

        yield ("hash160", b2h(hash160_c), None)

        if hash160_c and hash160_u:
            yield ("hash160_uncompressed", b2h(hash160_u), " uncompressed")

        address = network.address.for_p2pkh(hash160_c)
        yield ("address", address, "%s address" % network_name)
        yield ("%s_address" % network.symbol, address, "legacy")

        address = key.address(is_compressed=False)
        yield ("address_uncompressed", address, "%s address uncompressed" % network_name)
        yield ("%s_address_uncompressed" % network.symbol, address, "legacy")

        # don't print segwit addresses unless we're sure we have a compressed key
        if hash160_c and hasattr(network.address, "for_p2pkh_wit"):
            address_segwit = network.address.for_p2pkh_wit(hash160_c)
            if address_segwit:
                # this network seems to support segwit
                yield ("address_segwit", address_segwit, "%s segwit address" % network_name)
                yield ("%s_address_segwit" % network.symbol, address_segwit, "legacy")

                p2sh_script = network.contract.for_p2pkh_wit(hash160_c)
                p2s_address = network.address.for_p2s(p2sh_script)
                if p2s_address:
                    yield ("p2sh_segwit", p2s_address, None)

                p2sh_script_hex = b2h(p2sh_script)
                yield ("p2sh_segwit_script", p2sh_script_hex, " corresponding p2sh script")

    return f


def create_bitcoinish_network(symbol, network_name, subnet_name, **kwargs):
    # potential kwargs:
    #   tx, block, magic_header_hex, default_port, dns_bootstrap,
    #   wif_prefix_hex, address_prefix_hex, pay_to_script_prefix_hex
    #   bip32_prv_prefix_hex, bip32_pub_prefix_hex, sec_prefix, script_tools

    network = Network(symbol, network_name, subnet_name)

    generator = kwargs.get("generator", secp256k1_generator)
    kwargs.setdefault("sec_prefix", "%sSEC" % symbol.upper())
    KEYS_TO_H2B = ("bip32_prv_prefix bip32_pub_prefix wif_prefix address_prefix "
                   "pay_to_script_prefix sec_prefix magic_header").split()
    for k in KEYS_TO_H2B:
        k_hex = "%s_hex" % k
        if k_hex in kwargs:
            kwargs[k] = h2b(kwargs[k_hex])

    script_tools = kwargs.get("script_tools", BitcoinScriptTools)

    UI_KEYS = ("bip32_prv_prefix bip32_pub_prefix wif_prefix sec_prefix "
               "address_prefix pay_to_script_prefix bech32_hrp").split()
    ui_kwargs = {k: kwargs[k] for k in UI_KEYS if k in kwargs}

    _bip32_prv_prefix = ui_kwargs.get("bip32_prv_prefix")
    _bip32_pub_prefix = ui_kwargs.get("bip32_pub_prefix")
    _wif_prefix = ui_kwargs.get("wif_prefix")
    _sec_prefix = ui_kwargs.get("sec_prefix")

    def bip32_as_string(blob, as_private):
        prefix = _bip32_prv_prefix if as_private else _bip32_pub_prefix
        return b2a_hashed_base58(prefix + blob)

    def wif_for_blob(blob):
        return b2a_hashed_base58(_wif_prefix + blob)

    def sec_text_for_blob(blob):
        return _sec_prefix + b2h(blob)

    NetworkKey = Key.make_subclass(network=network, generator=generator)
    NetworkElectrumKey = ElectrumWallet.make_subclass(network=network, generator=generator)
    NetworkBIP32Node = BIP32Node.make_subclass(network=network, generator=generator)

    NETWORK_KEYS = "network_name subnet_name dns_bootstrap default_port magic_header".split()
    for k in NETWORK_KEYS:
        if k in kwargs:
            setattr(network, k, kwargs[k])

    network.Tx = network.tx = kwargs.get("tx") or Tx
    network.Block = network.block = kwargs.get("block") or Block.make_subclass(network.tx)

    streamer = standard_streamer(standard_parsing_functions(network.block, network.tx))

    network.message = API()
    network.message.parse, network.message.pack = make_parser_and_packer(
        streamer, standard_messages(), standard_message_post_unpacks(streamer))

    network.output_for_hwif = make_output_for_hwif(network)
    network.output_for_secret_exponent = make_output_for_secret_exponent(NetworkKey)
    network.output_for_public_pair = make_output_for_public_pair(NetworkKey, network)

    network.keychain = Keychain

    parse_api_class = kwargs.get("parse_api_class", ParseAPI)
    network.parse = parse_api_class(network, **ui_kwargs)

    network.contract = ContractAPI(network, script_tools)

    network.address = make_address_api(network.contract, **ui_kwargs)

    def keys_private(secret_exponent, is_compressed=True):
        return NetworkKey(secret_exponent=secret_exponent, is_compressed=is_compressed)

    def keys_public(item, is_compressed=None):
        if isinstance(item, tuple):
            if is_compressed is None:
                is_compressed = True
            # it's a public pair
            return NetworkKey(public_pair=item, is_compressed=is_compressed)
        if is_compressed is not None:
            raise ValueError("can't set is_compressed from sec")
        return NetworkKey.from_sec(item)

    network.keys = API()
    network.keys.private = keys_private
    network.keys.public = keys_public

    def electrum_seed(seed):
        return NetworkElectrumKey(initial_key=seed)

    def electrum_private(master_private_key):
        return NetworkElectrumKey(master_private_key=master_private_key)

    def electrum_public(master_public_key):
        return NetworkElectrumKey(master_public_key=master_public_key)

    def bip32_seed(seed):
        return NetworkBIP32Node.from_master_secret(seed)

    def bip32_deserialize(data):
        return NetworkBIP32Node.deserialize(data)

    network.keys.bip32_seed = bip32_seed
    network.keys.bip32_deserialize = bip32_deserialize

    network.keys.electrum_seed = electrum_seed
    network.keys.electrum_private = electrum_private
    network.keys.electrum_public = electrum_public
    network.keys.InvalidSecretExponentError = InvalidSecretExponentError
    network.keys.InvalidPublicPairError = InvalidPublicPairError

    network.msg = API()
    message_signer = MessageSigner(network, generator)
    network.msg.sign = message_signer.sign_message
    network.msg.verify = message_signer.verify_message
    network.msg.parse_signed = message_signer.parse_signed_message
    network.msg.hash_for_signing = message_signer.hash_for_signing
    network.msg.signature_for_message_hash = message_signer.signature_for_message_hash
    network.msg.pair_for_message_hash = message_signer.pair_for_message_hash
    network.script = script_tools

    network.bip32_as_string = bip32_as_string
    network.sec_text_for_blob = sec_text_for_blob
    network.wif_for_blob = wif_for_blob

    def network_build_hash160_lookup(iter):
        return build_hash160_lookup(iter, [generator])

    network.tx.solve = API()
    network.tx.solve.build_hash160_lookup = network_build_hash160_lookup
    network.tx.solve.build_p2sh_lookup = build_p2sh_lookup
    network.tx.solve.build_sec_lookup = build_sec_lookup

    network.validator = API()
    network.validator.ScriptError = ScriptError
    network.validator.ValidationFailureError = ValidationFailureError
    network.validator.errno = errno
    network.validator.flags = flags

    def my_create_tx(*args, **kwargs):
        return create_tx(network, *args, **kwargs)

    def my_sign_tx(*args, **kwargs):
        return sign_tx(network, *args, **kwargs)

    def my_create_signed_tx(*args, **kwargs):
        return create_signed_tx(network, *args, **kwargs)

    def my_split_with_remainder(*args, **kwargs):
        return split_with_remainder(network, *args, **kwargs)

    network.tx_utils = API()
    network.tx_utils.create_tx = my_create_tx
    network.tx_utils.sign_tx = my_sign_tx
    network.tx_utils.create_signed_tx = my_create_signed_tx
    network.tx_utils.split_with_remainder = my_split_with_remainder
    network.tx_utils.distribute_from_split_pool = distribute_from_split_pool

    network.annotate = Annotate(script_tools, network.address)

    network.who_signed = WhoSigned(script_tools, network.address, generator)

    network.str = parseable_str

    network.generator = generator

    return network



#!/usr/bin/env python

from setuptools import setup

with open("README.md", "rt") as fh:
    long_description = fh.read()

setup(
    name="pycoin",
    packages=[
        "pycoin",
        "pycoin.blockchain",
        "pycoin.cmds",
        "pycoin.coins",
        "pycoin.coins.bcash",
        "pycoin.coins.bgold",
        "pycoin.coins.bitcoin",
        "pycoin.coins.groestlcoin",
        "pycoin.coins.litecoin",
        "pycoin.contrib",
        "pycoin.convention",
        "pycoin.ecdsa",
        "pycoin.ecdsa.native",
        "pycoin.encoding",
        "pycoin.key",
        "pycoin.message",
        "pycoin.networks",
        "pycoin.satoshi",
        "pycoin.serialize",
        "pycoin.services",
        "pycoin.solve",
        "pycoin.symbols",
        "pycoin.vm",
        "pycoin.wallet"
    ],
    author="Richard Kiss",
    entry_points={
        'console_scripts':
            [
                'block = pycoin.cmds.block:main',
                'ku = pycoin.cmds.ku:main',
                'tx = pycoin.cmds.tx:main',
                'msg = pycoin.cmds.msg:main',
                'keychain = pycoin.cmds.keychain:main',
                'b58 = pycoin.cmds.b58:main',
                'coinc = pycoin.cmds.coinc:main',
            ]
        },
    author_email="him@richardkiss.com",
    url="https://github.com/richardkiss/pycoin",
    license="http://opensource.org/licenses/MIT",
    description="Utilities for Bitcoin and altcoin addresses and transaction manipulation.",
    long_description=long_description,
    long_description_content_type='text/markdown',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'License :: OSI Approved :: MIT License',
        'Topic :: Internet',
        'Topic :: Security :: Cryptography',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],)



import hashlib
import sqlite3
import textwrap

from collections import defaultdict

from pycoin.encoding.hash import hash160


class Keychain(object):
    def __init__(self, sqlite3_db=None):
        self._db = sqlite3_db or sqlite3.connect(":memory:")
        self._db.text_factory = type(b'')
        self._init_tables()
        self.clear_secrets()

    def commit(self):
        self._db.commit()

    def _exec_sql(self, sql, *args):
        c = self._db.cursor()
        c.execute(textwrap.dedent(sql), args)
        return c

    def _exec_sql_list(self, SQL):
        for sql in SQL:
            self._exec_sql(sql)

    def _init_table_hash160(self):
        self._exec_sql_list([
            "create table if not exists HASH160 (hash160 blob primary key, path text, fingerprint blob)",
        ])

    def _init_table_p2s(self):
        self._exec_sql_list([
            "create table if not exists P2S (hash160 blob primary key, hash256 blob, script blob)",
            "create index if not exists P2S_H256 on P2S (hash256)",
        ])

    def _init_tables(self):
        self._init_table_hash160()
        self._init_table_p2s()
        self.commit()

    def add_keys_path(self, keys, path):
        total = 0
        for key in keys:
            fingerprint = key.fingerprint()
            hash160 = key.subkey_for_path(path).hash160()
            self._exec_sql("insert or ignore into HASH160 values (?, ?, ?)", hash160, path, fingerprint)
            total += 1
        return total

    def add_key_paths(self, key, path_iterator=[""]):
        fingerprint = key.fingerprint()
        total = 0
        for path in path_iterator:
            hash160 = key.subkey_for_path(path).hash160()
            self._exec_sql("insert or ignore into HASH160 values (?, ?, ?)", hash160, path, fingerprint)
            total += 1
        return total

    def path_for_hash160(self, hash160):
        SQL = "select fingerprint, path from HASH160 where hash160 = ?"
        c = self._exec_sql(SQL, hash160)
        r = c.fetchone()
        if r is not None:
            return r[0], r[1].decode("utf8")

    def add_p2s_script(self, script):
        h160 = hash160(script)
        h256 = hashlib.sha256(script).digest()
        self._exec_sql("insert or ignore into P2S values (?, ?, ?)", h160, h256, script)

    def add_p2s_scripts(self, scripts):
        for script in scripts:
            self.add_p2s_script(script)
        self.commit()

    def p2s_for_hash(self, hash160or256):
        SQL = "select script from P2S where hash160 = ? or hash256 = ?"
        c = self._exec_sql(SQL, hash160or256, hash160or256)
        r = c.fetchone()
        if r is not None:
            return r[0]

    def _add_key_to_cache(self, key):
        secret_exponent = key.secret_exponent()
        public_pair = key.public_pair()
        for is_compressed in (True, False):
            hash160 = key.hash160(is_compressed=is_compressed)
            self._secret_exponent_cache[hash160] = (secret_exponent, public_pair, is_compressed, key._generator)

    def get(self, hash160, default=None):
        v = self.p2s_for_hash(hash160)
        if v:
            return v

        if hash160 not in self._secret_exponent_cache:
            v = self.path_for_hash160(hash160)
            if v:
                fingerprint, path = v
                for key in self._secrets.get(fingerprint, []):
                    subkey = key.subkey_for_path(path)
                    self._add_key_to_cache(subkey)

        return self._secret_exponent_cache.get(hash160, default)

    def add_secret(self, private_key):
        self._secrets[private_key.fingerprint()].add(private_key)
        self._add_key_to_cache(private_key)

    def add_secrets(self, private_keys):
        for key in private_keys:
            self.add_secret(key)

    def has_secrets(self):
        return len(self._secrets) + len(self._secret_exponent_cache) > 0

    def clear_secrets(self):
        self._secrets = defaultdict(set)
        self._secret_exponent_cache = {}

    def interested_hashes(self):
        SQL = "select hash160 from HASH160"
        c = self._exec_sql(SQL)
        for r in c:
            yield r[0]
        SQL = "select hash160, hash256 from P2S"
        c = self._exec_sql(SQL)
        for r in c:
            yield r[0]
            yield r[1]



import binascii


def h2b(h):
    """
    A version of binascii.unhexlify that accepts unicode. This is
    no longer necessary as of Python 3.3. But it doesn't hurt.

    Raises a ValueError on failure (unlike binascii.unhexlify, which
    raises a TypeError in Python 2 and a binascii.Error in Python 3).
    """
    try:
        return binascii.unhexlify(h.encode("ascii"))
    except Exception:
        raise ValueError("h2b failed on %s" % h)


def h2b_rev(h):
    return h2b(h)[::-1]


def b2h(the_bytes):
    return binascii.hexlify(the_bytes).decode("utf8")


def b2h_rev(the_bytes):
    return b2h(bytearray(reversed(the_bytes)))


class bytes_as_revhex(bytes):
    def __str__(self):
        return b2h_rev(self)

    def __repr__(self):
        return b2h_rev(self)


class bytes_as_hex(bytes):
    def __str__(self):
        return b2h(self)

    def __repr__(self):
        return b2h(self)



from .satoshi_int import parse_satoshi_int, stream_satoshi_int


def parse_satoshi_string(f):
    size = parse_satoshi_int(f)
    return f.read(size)


def stream_satoshi_string(f, v):
    stream_satoshi_int(f, len(v))
    f.write(v)



import unittest

from ctypes import byref, c_size_t, create_string_buffer

from pycoin.ecdsa.secp256k1 import secp256k1_generator
from pycoin.ecdsa.native.secp256k1 import libsecp256k1, SECP256K1_EC_UNCOMPRESSED
from pycoin.encoding.bytes32 import from_bytes_32, to_bytes_32
from pycoin.encoding.hexbytes import b2h
from pycoin.intbytes import int2byte, byte2int

from pycoin.ecdsa.secp256k1 import Generator, _p, _a, _b, _Gx, _Gy, _r

legacy_secp256k1_group = Generator(_p, _a, _b, (_Gx, _Gy), _r)


class ECDSATestCase(unittest.TestCase):

    def test_multiply_by_group_generator(self):
        self.assertEqual(1 * secp256k1_generator, (
            55066263022277343669578718895168534326250603453777594175500187360389116729240,
            32670510020758816978083085130507043184471273380659243275938904335757337482424)
        )
        self.assertEqual(2 * secp256k1_generator, (
            89565891926547004231252920425935692360644145829622209833684329913297188986597,
            12158399299693830322967808612713398636155367887041628176798871954788371653930)
        )
        self.assertEqual(
            secp256k1_generator *
            12158399299693830322967808612713398636155367887041628176798871954788371653930,
            (73503477726599187100887421812915680925855587149907858411827017692118332824920,
             27657251006027960104028534670901169416706551781681983309292004861017889370444)
        )

    def test_sign_verify_mutual_compatability(self):
        if libsecp256k1 is None:
            raise unittest.SkipTest("no libsecp256k1")
        ctx = libsecp256k1.ctx
        signature = create_string_buffer(64)
        sighash = to_bytes_32(1000)
        secret_key = to_bytes_32(100)

        public_key = create_string_buffer(64)
        r = libsecp256k1.secp256k1_ec_pubkey_create(ctx, public_key, secret_key)
        self.assertEqual(r, 1)
        self.assertEqual(
            b2h(public_key),
            '880f50f7ceb4210289266a40b306e33ef52bb75f834c172e65175e3ce2ac3bed'
            '6e2835e3d57ae1fcd0954808be17bd97bf871f7a8a5edadcffcc8812576f7ae5'
        )

        r = libsecp256k1.secp256k1_ecdsa_sign(ctx, signature, sighash, secret_key, None, None)
        self.assertEqual(r, 1)

        r = libsecp256k1.secp256k1_ecdsa_verify(ctx, signature, sighash, public_key)
        self.assertEqual(r, 1)

        signature1 = signature[:-1] + int2byte(byte2int(signature[-1]) ^ 1)
        r = libsecp256k1.secp256k1_ecdsa_verify(ctx, signature1, sighash, public_key)
        self.assertEqual(r, 0)

    def test_sign(self):
        if libsecp256k1 is None:
            raise unittest.SkipTest("no libsecp256k1")
        ctx = libsecp256k1.ctx
        sighash = to_bytes_32(1000)
        secret_key = to_bytes_32(100)

        public_key = create_string_buffer(64)
        r = libsecp256k1.secp256k1_ec_pubkey_create(ctx, public_key, secret_key)
        self.assertEqual(r, 1)
        self.assertEqual(
            b2h(public_key),
            '880f50f7ceb4210289266a40b306e33ef52bb75f834c172e65175e3ce2ac3bed'
            '6e2835e3d57ae1fcd0954808be17bd97bf871f7a8a5edadcffcc8812576f7ae5'
        )

        signature = create_string_buffer(64)
        r = libsecp256k1.secp256k1_ecdsa_sign(ctx, signature, sighash, secret_key, None, None)
        self.assertEqual(r, 1)

        compact_signature = create_string_buffer(64)
        libsecp256k1.secp256k1_ecdsa_signature_serialize_compact(ctx, compact_signature, signature)
        r = from_bytes_32(compact_signature[:32])
        s = from_bytes_32(compact_signature[32:])
        signature = (r, s)

        pubkey_size = c_size_t(65)
        pubkey_serialized = create_string_buffer(65)
        libsecp256k1.secp256k1_ec_pubkey_serialize(
            ctx, pubkey_serialized, byref(pubkey_size), public_key, SECP256K1_EC_UNCOMPRESSED)
        x = from_bytes_32(pubkey_serialized[1:33])
        y = from_bytes_32(pubkey_serialized[33:])

        legacy_secp256k1_group.verify((x, y), 1000, signature)

    def test_verify(self):
        public_pair = secp256k1_generator * 1
        self.assertEqual(public_pair, (
            55066263022277343669578718895168534326250603453777594175500187360389116729240,
            32670510020758816978083085130507043184471273380659243275938904335757337482424)
        )
        hash_value = 1
        sig = (46340862580836590753275244201733144181782255593078084106116359912084275628184,
               81369331955758484632176499244870227132558660296342819670803726373940306621624)
        r = secp256k1_generator.verify(public_pair, hash_value, sig)
        self.assertEqual(r, True)


if __name__ == '__main__':
    unittest.main()



import unittest

from pycoin.key.subpaths import subpaths_for_path_range
from pycoin.symbols.btc import network


class KeychainTest(unittest.TestCase):

    def test_keychain(self):
        keychain = network.keychain()
        bip32_list = [network.keys.bip32_seed(_) for _ in [b"foo", b"bar"]]
        for bip32 in bip32_list:
            keychain.add_key_paths(bip32.public_copy(), subpaths_for_path_range("0-1/0-10"))
        keychain.add_secrets(bip32_list)
        for bip32 in bip32_list:
            for path in ["0/5", "1/2", "0/9"]:
                subkey = bip32.subkey_for_path("0/5")
                v = keychain.get(subkey.hash160())
                self.assertEqual(v[0], subkey.secret_exponent())
        v = keychain.get(b'0' * 32)
        self.assertEqual(v, None)



import functools
import struct

from pycoin.intbytes import iterbytes
from pycoin.satoshi.satoshi_struct import parse_struct
from pycoin.encoding.hexbytes import h2b


IP4_HEADER = h2b("00000000000000000000FFFF")


def ip_bin_to_ip6_addr(ip_bin):
    return ":".join("%x" % v for v in struct.unpack(">HHHHHHHH", ip_bin))


def ip_bin_to_ip4_addr(ip_bin):
    return "%d.%d.%d.%d" % tuple(iterbytes(ip_bin[-4:]))


@functools.total_ordering
class PeerAddress(object):
    def __init__(self, services, ip_bin, port):
        self.services = int(services)
        assert isinstance(ip_bin, bytes)
        if len(ip_bin) == 4:
            ip_bin = IP4_HEADER + ip_bin
        assert len(ip_bin) == 16
        self.ip_bin = ip_bin
        self.port = port

    def __repr__(self):
        return "%s/%d" % (self.host(), self.port)

    def host(self):
        if self.ip_bin.startswith(IP4_HEADER):
            return ip_bin_to_ip4_addr(self.ip_bin[-4:])
        return ip_bin_to_ip6_addr(self.ip_bin)

    def stream(self, f):
        f.write(struct.pack("<Q", self.services))
        f.write(self.ip_bin)
        f.write(struct.pack("!H", self.port))

    @classmethod
    def parse(self, f):
        services, ip_bin, port = parse_struct("Q@h", f)
        self.ip_bin = ip_bin
        return self(services, self.ip_bin, port)

    def __lt__(self, other):
        return (self.ip_bin, self.port, self.services) < (other.ip_bin, other.port, other.services)

    def __eq__(self, other):
        return self.services == other.services and \
            self.ip_bin == other.ip_bin and self.port == other.port



from pycoin.networks.bitcoinish import create_bitcoinish_network

network = create_bitcoinish_network(
    symbol="XDT", network_name="Dogecoin", subnet_name="testnet",
    wif_prefix_hex="f1", address_prefix_hex="71", pay_to_script_prefix_hex="c4",
    bip32_prv_prefix_hex="0432a9a8", bip32_pub_prefix_hex="0432a243")



"""
Utilities to convert to and from base58.
"""

from .base_conversion import from_long, to_long, EncodingError
from .hash import double_sha256
from ..intbytes import iterbytes


BASE58_ALPHABET = b'123456789ABCDEFGHJKLMNPQRSTUVWXYZabcdefghijkmnopqrstuvwxyz'
BASE58_BASE = len(BASE58_ALPHABET)
BASE58_LOOKUP = dict((c, i) for i, c in enumerate(BASE58_ALPHABET))


def b2a_base58(s):
    """Convert binary to base58 using BASE58_ALPHABET. Like Bitcoin addresses."""
    v, prefix = to_long(256, lambda x: x, iterbytes(s))
    s = from_long(v, prefix, BASE58_BASE, lambda v: BASE58_ALPHABET[v])
    return s.decode("utf8")


def a2b_base58(s):
    """Convert base58 to binary using BASE58_ALPHABET."""
    v, prefix = to_long(BASE58_BASE, lambda c: BASE58_LOOKUP[c], s.encode("utf8"))
    return from_long(v, prefix, 256, lambda x: x)


def b2a_hashed_base58(data):
    """
    A "hashed_base58" structure is a base58 integer (which looks like a string)
    with four bytes of hash data at the end. Bitcoin does this in several places,
    including Bitcoin addresses.

    This function turns data (of type "bytes") into its hashed_base58 equivalent.
    """
    return b2a_base58(data + double_sha256(data)[:4])


def a2b_hashed_base58(s):
    """
    If the passed string is hashed_base58, return the binary data.
    Otherwise raises an EncodingError.
    """
    data = a2b_base58(s)
    data, the_hash = data[:-4], data[-4:]
    if double_sha256(data)[:4] == the_hash:
        return data
    raise EncodingError("hashed base58 has bad checksum %s" % s)


def is_hashed_base58_valid(base58):
    """Return True if and only if base58 is valid hashed_base58."""
    try:
        a2b_hashed_base58(base58)
    except EncodingError:
        return False
    return True


"""
The MIT License (MIT)

Copyright (c) 2013 by Richard Kiss

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
"""



from pycoin.satoshi import errno
from pycoin.satoshi.flags import VERIFY_MINIMALDATA
from pycoin.vm.ConditionalStack import ConditionalStack

from pycoin.coins.SolutionChecker import ScriptError


def conditional_error_f(msg):
    raise ScriptError(msg, errno.UNBALANCED_CONDITIONAL)


class VM(object):
    MAX_SCRIPT_LENGTH = 10000
    MAX_BLOB_LENGTH = 520
    MAX_OP_COUNT = 201
    MAX_STACK_SIZE = 1000

    VM_FALSE = b''
    VM_TRUE = b'\1'

    ConditionalStack = ConditionalStack

    def __init__(self, script, tx_context, signature_for_hash_type_f, flags, initial_stack=None, traceback_f=None):
        self.pc = 0
        self.script = script
        self.tx_context = tx_context
        self.stack = initial_stack or list()
        self.altstack = list()
        self.conditional_stack = self.ConditionalStack(conditional_error_f)
        self.op_count = 0
        self.begin_code_hash = 0
        self.flags = flags
        self.traceback_f = traceback_f
        self.signature_for_hash_type_f = signature_for_hash_type_f

    def append(self, a):
        self.stack.append(a)

    def pop(self, *args, **kwargs):
        try:
            return self.stack.pop(*args, **kwargs)
        except IndexError:
            raise ScriptError("pop from empty stack", errno.INVALID_STACK_OPERATION)

    def __getitem__(self, *args, **kwargs):
        try:
            return self.stack.__getitem__(*args, **kwargs)
        except IndexError:
            raise ScriptError("getitem out of range", errno.INVALID_STACK_OPERATION)

    def pop_int(self):
        raise NotImplemented

    def pop_nonnegative(self):
        v = self.pop_int()
        if v < 0:
            raise ScriptError("unexpectedly got negative value", errno.INVALID_STACK_OPERATION)
        return v

    def push_int(self, v):
        raise NotImplemented

    @classmethod
    def bool_from_script_bytes(class_, v, require_minimal=False):
        raise NotImplemented

    @classmethod
    def bool_to_script_bytes(class_, v):
        raise NotImplemented

    @classmethod
    def generator_for_signature_type(class_, signature_type):
        raise NotImplemented

    def eval_script(self):
        if len(self.script) > self.MAX_SCRIPT_LENGTH:
            raise ScriptError("script too long", errno.SCRIPT_SIZE)

        f = getattr(self.traceback_f, "prelaunch", None)
        if f:
            f(self)

        while self.pc < len(self.script):
            self.eval_instruction()

        f = getattr(self.traceback_f, "postscript", None)
        if f:
            f(self)

        self.post_script_check()
        return self.stack

    def eval_instruction(self):
        all_if_true = self.conditional_stack.all_if_true()

        # don't actually check for minimal data unless data will be pushed onto the stack
        verify_minimal_data = self.flags & VERIFY_MINIMALDATA and all_if_true
        opcode, data, pc, is_ok = self.ScriptStreamer.get_opcode(
            self.script, self.pc, verify_minimal_data=verify_minimal_data)
        if not is_ok:
            raise ScriptError("malformed data", errno.BAD_OPCODE)
        if data and len(data) > self.MAX_BLOB_LENGTH:
            raise ScriptError("pushing too much data onto stack", errno.PUSH_SIZE)

        if data is None:
            self.op_count += 1

        self.check_stack_size()

        f = self.INSTRUCTION_LOOKUP[opcode]
        if self.traceback_f:
            f = self.traceback_f(opcode, data, pc, self) or f

        if data is not None and all_if_true:
            self.stack.append(data)

        self.pc = pc

        if all_if_true or getattr(f, "outside_conditional", False):
            f(self)

        if self.op_count > self.MAX_OP_COUNT:
            raise ScriptError("script contains too many operations", errno.OP_COUNT)

    def check_stack_size(self):
        if len(self.stack) + len(self.altstack) > self.MAX_STACK_SIZE:
            raise ScriptError("stack has > %d items" % self.MAX_STACK_SIZE, errno.STACK_SIZE)

    def post_script_check(self):
        self.conditional_stack.check_final_state()
        self.check_stack_size()



import io
import json
import warnings

from .agent import request, urlencode, urlopen

from pycoin.coins.bitcoin.Tx import Tx
from pycoin.encoding.hexbytes import b2h, h2b, b2h_rev


class BlockchainInfoProvider(object):
    def __init__(self, netcode):
        if netcode == 'BTC':
            self.api_domain = "https://blockchain.info"
        elif netcode == "XTN":
            self.api_domain = "https://testnet.blockchain.info"
        else:
            raise ValueError("unsupported netcode %s" % netcode)

    def tx_for_tx_hash(self, tx_hash):
        "Get a Tx by its hash."
        URL = self.api_domain + ("/rawtx/%s?format=hex" % b2h_rev(tx_hash))
        tx = Tx.from_hex(urlopen(URL).read().decode("utf8"))
        return tx

    def payments_for_address(self, address):
        "return an array of (TX ids, net_payment)"
        URL = self.api_domain + ("/address/%s?format=json" % address)
        d = urlopen(URL).read()
        json_response = json.loads(d.decode("utf8"))
        response = []
        for tx in json_response.get("txs", []):
            total_out = 0
            for tx_out in tx.get("out", []):
                if tx_out.get("addr") == address:
                    total_out += tx_out.get("value", 0)
            if total_out > 0:
                response.append((tx.get("hash"), total_out))
        return response

    def spendables_for_address(self, address):
        """
        Return a list of Spendable objects for the
        given bitcoin address.
        """
        URL = self.api_domain + "/unspent?active=%s" % address
        r = json.loads(urlopen(URL).read().decode("utf8"))
        spendables = []
        for u in r["unspent_outputs"]:
            coin_value = u["value"]
            script = h2b(u["script"])
            previous_hash = h2b(u["tx_hash"])
            previous_index = u["tx_output_n"]
            spendables.append(Tx.Spendable(coin_value, script, previous_hash, previous_index))
        return spendables

    def broadcast_tx(self, tx):
        s = io.BytesIO()
        tx.stream(s)
        tx_as_hex = b2h(s.getvalue())
        data = urlencode(dict(tx=tx_as_hex)).encode("utf8")
        URL = self.api_domain + "/pushtx"
        try:
            d = urlopen(URL, data=data).read()
            return d
        except request.HTTPError as ex:
            try:
                d = ex.read()
                ex.message = d
            except Exception:
                pass
            raise ex


def send_tx(self, tx):
    warnings.warn("use BlockchainInfoProvider.broadcast_tx instead of send_tx",
                  category=DeprecationWarning)
    return BlockchainInfoProvider().broadcast_tx(tx)



import collections
import itertools

from pycoin.encoding.hash import hash160
from pycoin.encoding.hexbytes import b2h
from pycoin.encoding.sec import is_sec_compressed, public_pair_to_hash160_sec
from pycoin.intbytes import byte2int
from pycoin.satoshi.flags import SIGHASH_ALL, SIGHASH_NONE, SIGHASH_SINGLE, SIGHASH_ANYONECANPAY, SIGHASH_FORKID
from pycoin.satoshi.checksigops import parse_signature_blob
from pycoin.coins.SolutionChecker import ScriptError


class Annotate(object):
    BIT_LIST = [(SIGHASH_ANYONECANPAY, "SIGHASH_ANYONECANPAY"), (SIGHASH_FORKID, "SIGHASH_FORKID")]
    BASE_LOOKUP = {SIGHASH_ALL: "SIGHASH_ALL", SIGHASH_SINGLE: "SIGHASH_SINGLE", SIGHASH_NONE: "SIGHASH_NONE"}

    def __init__(self, script_tools, address_api):
        self._script_tools = script_tools
        self._address = address_api
        for _ in "EQUAL HASH160 CHECKSIG CHECKSIGVERIFY CHECKMULTISIG CHECKMULTISIGVERIFY".split():
            setattr(self, "OP_%s" % _, byte2int(self._script_tools.compile('OP_%s' % _)))

    def sighash_type_to_string(self, sighash_type):
        v = sighash_type
        flag_bit_list = []
        for flag_bit, flag_name in self.BIT_LIST:
            if v & flag_bit:
                v &= ~flag_bit
                flag_bit_list.append(flag_name)
        base_type = self.BASE_LOOKUP.get(v, "SIGHASH_UNKNOWN")
        return "".join([base_type] + [" | %s" % s for s in flag_bit_list])

    def instruction_for_opcode(self, opcode, data):
        if data is None or len(data) == 0:
            return self._script_tools.disassemble_for_opcode_data(opcode, data)
        return "[PUSH_%d] %s" % (opcode, b2h(data))

    def annotate_pubkey(self, blob, da):
        is_compressed = is_sec_compressed(blob)
        address = self._address.for_p2pkh(hash160(blob))
        da[blob].append("SEC for %scompressed %s" % ("" if is_compressed else "un", address))

    def annotate_signature(self, blob, da, vmc):
        lst = da[blob]
        try:
            sig_pair, sig_type = parse_signature_blob(blob)
        except ValueError:
            return
        lst.append("r: {0:#066x}".format(sig_pair[0]))
        lst.append("s: {0:#066x}".format(sig_pair[1]))
        sig_hash = vmc.signature_for_hash_type_f(sig_type, [blob], vmc)
        lst.append("z: {0:#066x}".format(sig_hash))
        lst.append("signature type %s" % self.sighash_type_to_string(sig_type))
        addresses = []
        generator = vmc.generator_for_signature_type(sig_type)
        pairs = generator.possible_public_pairs_for_signature(sig_hash, sig_pair)
        for pair in pairs:
            for comp in (True, False):
                hash160 = public_pair_to_hash160_sec(pair, compressed=comp)
                address = self._address.for_p2pkh(hash160)
                addresses.append(address)
        lst.append(" sig for %s" % " ".join(addresses))

    def annotate_checksig(self, vmc, da):
        s = list(vmc.stack)
        try:
            self.annotate_pubkey(vmc.pop(), da)
            self.annotate_signature(vmc.pop(), da, vmc)
        except (IndexError, ValueError):
            pass
        vmc.stack = s

    def annotate_checkmultisig(self, vmc, da):
        s = list(vmc.stack)
        try:
            key_count = vmc.pop_int()
            while key_count > 0:
                key_count -= 1
                self.annotate_pubkey(vmc.pop(), da)

            signature_count = vmc.pop_int()
            while signature_count > 0:
                signature_count -= 1
                self.annotate_signature(vmc.pop(), da, vmc)
        except IndexError:
            pass
        vmc.stack = s

    def annotate_scripts(self, tx, tx_in_idx):
        "return list of pre_annotations, pc, opcode, instruction, post_annotations"
        # input_annotations_f, output_annotations_f = annotation_f_for_scripts(tx, tx_in_idx)

        data_annotations = collections.defaultdict(list)

        def traceback_f(opcode, data, pc, vmc):
            if opcode in (self.OP_CHECKSIG, self.OP_CHECKSIGVERIFY):
                self.annotate_checksig(vmc, data_annotations)
            if opcode in (self.OP_CHECKMULTISIG, self.OP_CHECKMULTISIGVERIFY):
                self.annotate_checkmultisig(vmc, data_annotations)
            return

        try:
            tx.check_solution(tx_in_idx, traceback_f=traceback_f)
        except ScriptError:
            pass

        r = []

        def traceback_f(opcode, data, pc, vmc):
            a0 = []
            if vmc.pc == 0:
                if vmc.is_solution_script:
                    a0.append("--- SIGNATURE SCRIPT START")
                else:
                    a0.append("--- PUBLIC KEY SCRIPT START")
            r.append((a0, vmc.pc, opcode, self.instruction_for_opcode(opcode, data), data_annotations[data]))

        try:
            tx.check_solution(tx_in_idx, traceback_f=traceback_f)
        except ScriptError:
            pass

        # the script may have ended early, so let's just double-check
        try:
            for idx, (opcode, data, pc, new_pc) in enumerate(itertools.chain(
                self._script_tools.get_opcodes(tx.unspents[tx_in_idx].script),
                    self._script_tools.get_opcodes(tx.txs_in[tx_in_idx].script))):
                if idx >= len(r):
                    r.append(([], pc, opcode, self.instruction_for_opcode(opcode, data), []))
        except IndexError:
            pass

        return r

    def annotate_spendable(self, tx_class, spendable):
        txs_in = [tx_class.TxIn(b'1' * 32, 0)]
        fake_spend_tx = tx_class(1, txs_in, [])
        fake_spend_tx.set_unspents([spendable])
        return self.annotate_scripts(fake_spend_tx, 0)



import io
import unittest

from pycoin.encoding.hexbytes import h2b
from pycoin.message.make_parser_and_packer import (
    make_parser_and_packer, standard_messages, standard_message_post_unpacks,
    standard_parsing_functions, standard_streamer
)
from pycoin.message.PeerAddress import ip_bin_to_ip4_addr, ip_bin_to_ip6_addr, PeerAddress, IP4_HEADER
from pycoin.message.InvItem import InvItem, ITEM_TYPE_BLOCK, ITEM_TYPE_TX
from pycoin.symbols.btc import network


def to_bin(obj):
    f = io.BytesIO()
    obj.stream(f)
    return f.getvalue()


def from_bin(cls, blob):
    f = io.BytesIO(blob)
    return cls.parse(f)


class MessageTest(unittest.TestCase):

    def test_make_parser_and_packer(self):
        streamer = standard_streamer(standard_parsing_functions(network.block, network.tx))
        btc_parser, btc_packer = make_parser_and_packer(
            streamer, standard_messages(), standard_message_post_unpacks(streamer))
        parser, packer = make_parser_and_packer(streamer, standard_messages(), standard_message_post_unpacks(streamer))
        cases = [("verack", {}), ("ping", dict(nonce=1929)), ("pong", dict(nonce=18373))]
        for msg_name, kwargs in cases:
            data = packer(msg_name, **kwargs)
            v = parser(msg_name, data)
            assert v == kwargs

    def test_ipv6(self):
        self.assertEqual(
            ip_bin_to_ip6_addr(h2b("2607f8b04006080a000000000000200e")),
            "2607:f8b0:4006:80a:0:0:0:200e"
        )

    def test_ipv4(self):
        ip4_bin = h2b("c0a80163")
        self.assertEqual(ip_bin_to_ip4_addr(ip4_bin), "192.168.1.99")

    def test_PeerAddress(self):
        pa = PeerAddress(188, IP4_HEADER + h2b("c0a80163"), 8333)
        pa_bytes = to_bin(pa)
        pa1 = from_bin(PeerAddress, pa_bytes)
        self.assertEqual(pa, pa1)

        pa2 = PeerAddress(188, IP4_HEADER + h2b("c0a80162"), 8333)
        self.assertTrue(pa1 > pa2)
        self.assertTrue(pa1 >= pa2)
        self.assertTrue(pa2 < pa1)
        self.assertTrue(pa2 <= pa1)
        self.assertNotEqual(pa2, pa1)
        self.assertNotEqual(pa1, pa2)
        self.assertEqual(pa1.host(), "192.168.1.99")
        self.assertEqual(repr(pa1), "192.168.1.99/8333")

        pa_v6 = PeerAddress(945, h2b("2607f8b04006080a000000000000200e"), 8333)
        self.assertEqual(pa_v6.host(), "2607:f8b0:4006:80a:0:0:0:200e")

    def test_InvItem(self):
        ii0 = InvItem(ITEM_TYPE_TX, b'\0' * 32)
        ii1 = InvItem(ITEM_TYPE_BLOCK, b'\0' * 32)
        self.assertTrue(ii0 < ii1)
        self.assertTrue(ii1 > ii0)
        self.assertTrue(ii0 <= ii1)
        self.assertTrue(ii1 >= ii0)
        self.assertTrue(ii1 != ii0)
        self.assertFalse(ii1 == ii0)
        self.assertTrue(hash(ii1) != hash(ii0))

        for ii in (ii0, ii1):
            blob = to_bin(ii)
            ii_prime = from_bin(InvItem, blob)
            self.assertEqual(ii, ii_prime)



from pycoin.networks.bitcoinish import create_bitcoinish_network

network = create_bitcoinish_network(
    symbol="TBTX", network_name="BitCore", subnet_name="testnet3",
    wif_prefix_hex="EF", sec_prefix="TBTXSEC:", address_prefix_hex="6F", pay_to_script_prefix_hex="C4",
    bip32_prv_prefix_hex="04358394", bip32_pub_prefix_hex="043587CF",
    magic_header_hex="FDD2C8F1", default_port=8666,
    dns_bootstrap=[
        "188.68.52.172", "51.15.84.165"
    ])



#!/usr/bin/env python

import argparse
import datetime

from .dump import dump_tx
from pycoin.encoding.hexbytes import b2h, b2h_rev
from pycoin.networks.default import get_current_netcode
from pycoin.networks.registry import network_for_netcode
from pycoin.serialize import stream_to_bytes


def dump_block(output, block, network):
    blob = stream_to_bytes(block.stream)
    output.append("%d bytes   block hash %s" % (len(blob), block.id()))
    output.append("version %d" % block.version)
    output.append("prior block hash %s" % b2h_rev(block.previous_block_hash))
    output.append("merkle root %s" % b2h(block.merkle_root))
    output.append("timestamp %s" % datetime.datetime.utcfromtimestamp(block.timestamp).isoformat())
    output.append("difficulty %d" % block.difficulty)
    output.append("nonce %s" % block.nonce)
    output.append("%d transaction%s" % (len(block.txs), "s" if len(block.txs) != 1 else ""))
    for idx, tx in enumerate(block.txs):
        output.append("Tx #%d:" % idx)
        dump_tx(
            output, tx, network=network, verbose_signature=False, disassembly_level=0, do_trace=False, use_pdb=False)
    output.append("")


def create_parser():
    parser = argparse.ArgumentParser(description="Dump a block in human-readable form.")
    parser.add_argument('-n', "--network", default=get_current_netcode(), type=network_for_netcode,
                        help=('Default network code (environment variable PYCOIN_DEFAULT_NETCODE '
                              'or "BTC"=Bitcoin mainnet if unset'))
    parser.add_argument("block_file", nargs="+", type=argparse.FileType('rb'),
                        help='The file containing the binary block.')
    return parser


def block(args, parser):
    network = args.network
    for f in args.block_file:
        block = network.block.parse(f)
        output = []
        dump_block(output, block, network)

        for line in output:
            print(line)


def main():
    parser = create_parser()
    args = parser.parse_args()
    block(args, parser)


if __name__ == '__main__':
    main()



from pycoin import version

try:
    import urllib2 as request
    from urllib import urlencode  # noqa
except ImportError:
    from urllib import request
    from urllib.parse import urlencode  # noqa


PYCOIN_AGENT = 'pycoin/%s' % version


def urlopen(url, data=None):
    req = request.Request(url, data=data)
    req.add_header('User-agent', PYCOIN_AGENT)
    return request.urlopen(req)



import hashlib

from . import errno
from pycoin.coins.SolutionChecker import ScriptError

from ..encoding.hash import hash160, double_sha256, ripemd160


def do_OP_NOP(s):
    pass


for i in range(1, 11):
    exec("def do_OP_NOP%d(s): pass" % i)


def do_OP_VER(stack):
    raise ScriptError("OP_VER encountered", errno.BAD_OPCODE)


def do_OP_RESERVED1(stack):
    raise ScriptError("OP_RESERVED1 encountered", errno.BAD_OPCODE)


def do_OP_RESERVED2(stack):
    raise ScriptError("OP_RESERVED2 encountered", errno.BAD_OPCODE)


def do_OP_RETURN(stack):
    raise ScriptError("OP_RETURN encountered", errno.OP_RETURN)


def do_OP_2DROP(stack):
    stack.pop()
    stack.pop()


def do_OP_2DUP(stack):
    #  (x1 x2 -- x1 x2 x1 x2)
    stack.append(stack[-2])
    stack.append(stack[-2])


def do_OP_3DUP(stack):
    #  (x1 x2 x3 -- x1 x2 x3 x1 x2 x3)
    stack.append(stack[-3])
    stack.append(stack[-3])
    stack.append(stack[-3])


def do_OP_2OVER(stack):
    #  (x1 x2 x3 x4 -- x1 x2 x3 x4 x1 x2)
    stack.append(stack[-4])
    stack.append(stack[-4])


def do_OP_2ROT(stack):
    # (1, 2, 3, 4, 5, 6 -- 3, 4, 5, 6, 1, 2)
    stack.append(stack.pop(-6))
    stack.append(stack.pop(-6))


def do_OP_2SWAP(stack):
    stack.append(stack.pop(-4))
    stack.append(stack.pop(-4))


def do_OP_IFDUP(stack):
    if stack[-1]:
        stack.append(stack[-1])


def do_OP_DROP(stack):
    stack.pop()


def do_OP_DUP(stack):
    stack.append(stack[-1])


def do_OP_NIP(stack):
    v = stack.pop()
    stack.pop()
    stack.append(v)


def do_OP_OVER(stack):
    stack.append(stack[-2])


def do_OP_ROT(stack):
    stack.append(stack.pop(-3))


def do_OP_SWAP(stack):
    stack.append(stack.pop(-2))


def do_OP_TUCK(stack):
    v1 = stack.pop()
    v2 = stack.pop()
    stack.append(v1)
    stack.append(v2)
    stack.append(v1)


def do_OP_CAT(stack):
    v1 = stack.pop()
    v2 = stack.pop()
    stack.append(v2 + v1)


def do_OP_RIPEMD160(stack):
    stack.append(ripemd160(stack.pop()).digest())


def do_OP_SHA1(stack):
    stack.append(hashlib.sha1(stack.pop()).digest())


def do_OP_SHA256(stack):
    stack.append(hashlib.sha256(stack.pop()).digest())


def do_OP_HASH160(stack):
    stack.append(hash160(stack.pop()))


def do_OP_HASH256(stack):
    stack.append(double_sha256(stack.pop()))


"""
The MIT License (MIT)

Copyright (c) 2013 by Richard Kiss

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
"""



#!/usr/bin/env python

# This script shows you how to create a "2-of-3" multisig address.
# It requires BIP32 private key file.

import os
import sys

from pycoin.encoding.hexbytes import b2h
from pycoin.symbols.btc import network


def main():
    if len(sys.argv) != 2:
        print("usage: %s bip32_key_file" % sys.argv[0])
        sys.exit(-1)
    with open(sys.argv[1], "r") as f:
        hwif = f.readline().strip()

    # turn the bip32 text into a BIP32Node object
    BIP32_KEY = network.parse(hwif)

    # create three sec_keys (these are public keys, streamed using the SEC format)

    SEC_0 = BIP32_KEY.subkey_for_path("0/0/0").sec()
    SEC_1 = BIP32_KEY.subkey_for_path("0/1/0").sec()
    SEC_2 = BIP32_KEY.subkey_for_path("0/2/0").sec()

    public_key_sec_list = [SEC_0, SEC_1, SEC_2]

    # create the 2-of-3 multisig script
    # any 2 signatures can release the funds
    pay_to_multisig_script = network.contract.for_multisig(2, public_key_sec_list)

    # create a "2-of-3" multisig address_for_multisig
    the_address = network.address.for_p2s(pay_to_multisig_script)

    print("Here is your pay 2-of-3 address: %s" % the_address)

    print("Here is the pay 2-of-3 script: %s" % b2h(pay_to_multisig_script))
    print("The hex script should go into p2sh_lookup.hex")

    base_dir = os.path.abspath(os.path.dirname(sys.argv[1]))
    print("The three WIFs are written into %s as wif0, wif1 and wif2" % base_dir)
    for i in range(3):
        wif = BIP32_KEY.subkey_for_path("0/%d/0" % i).wif()
        with open(os.path.join(base_dir, "wif%d" % i), "w") as f:
            f.write(wif)


if __name__ == '__main__':
    main()
